@@ -1,8 +1,9 @@
-# Copyright (C) 2006 by Canonical Ltd
+# Copyright (C) 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,13 +14,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Benchmarks of bzr commit."""
+"""Benchmarks of bzr rocks."""
 
 
 from bzrlib.benchmarks import Benchmark
 
 
 class RocksBenchmark(Benchmark):
+    """Benchmarks for ``'bzr rocks'``"""
 
     def test_rocks(self):
         """Test the startup overhead by running a do-nothing command"""
