@@ -21,7 +21,7 @@ import os
 
 from bzrlib import errors
 from bzrlib.bundle.serializer import (BundleSerializer,
-                                      BUNDLE_HEADER,
+                                      _get_bundle_header,
                                      )
 from bzrlib.bundle.serializer import binary_diff
 from bzrlib.bundle.bundle_data import (RevisionInfo, BundleInfo, BundleTree)
@@ -29,7 +29,6 @@ from bzrlib.diff import internal_diff
 from bzrlib.osutils import pathjoin
 from bzrlib.progress import DummyProgress
 from bzrlib.revision import NULL_REVISION
-from bzrlib.rio import RioWriter, read_stanzas
 import bzrlib.ui
 from bzrlib.testament import StrictTestament
 from bzrlib.timestamp import (
@@ -129,21 +128,33 @@ class BundleSerializerV08(BundleSerializer):
         finally:
             source.unlock()
 
+    def write_bundle(self, repository, target, base, fileobj):
+        return self._write_bundle(repository, target, base, fileobj)
+
     def _write_main_header(self):
         """Write the header for the changes"""
         f = self.to_file
-        f.write(BUNDLE_HEADER)
-        f.write('0.8\n')
+        f.write(_get_bundle_header('0.8'))
         f.write('#\n')
 
-    def _write(self, key, value, indent=1):
-        """Write out meta information, with proper indenting, etc"""
+    def _write(self, key, value, indent=1, trailing_space_when_empty=False):
+        """Write out meta information, with proper indenting, etc.
+
+        :param trailing_space_when_empty: To work around a bug in earlier
+            bundle readers, when writing an empty property, we use "prop: \n"
+            rather than writing "prop:\n".
+            If this parameter is True, and value is the empty string, we will
+            write an extra space.
+        """
         assert indent > 0, 'indentation must be greater than 0'
         f = self.to_file
         f.write('#' + (' ' * indent))
         f.write(key.encode('utf-8'))
         if not value:
-            f.write(':\n')
+            if trailing_space_when_empty and value == '':
+                f.write(': \n')
+            else:
+                f.write(':\n')
         elif isinstance(value, str):
             f.write(': ')
             f.write(value)
@@ -226,8 +237,9 @@ class BundleSerializerV08(BundleSerializer):
             w('base id', base_rev)
         if rev.properties:
             self._write('properties', None, indent=1)
-            for name, value in rev.properties.items():
-                self._write(name, value, indent=3)
+            for name, value in sorted(rev.properties.items()):
+                self._write(name, value, indent=3,
+                            trailing_space_when_empty=True)
         
         # Add an extra blank space at the end
         self.to_file.write('\n')
@@ -529,7 +541,6 @@ class BundleReader(object):
                 self._next().next()
                 break
 
-
 class BundleInfo08(BundleInfo):
 
     def _update_tree(self, bundle_tree, revision_id):
