@@ -51,8 +51,6 @@ from bzrlib import (
 from bzrlib.symbol_versioning import (
     deprecated_function,
     deprecated_method,
-    zero_eight,
-    zero_eleven,
     )
 # Compatibility
 from bzrlib.option import Option
@@ -87,6 +85,8 @@ def register_command(cmd, decorate=False):
     else:
         trace.log_error('Two plugins defined the same command: %r' % k)
         trace.log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
+        trace.log_error('Previously this command was registered from %r' %
+                        sys.modules[plugin_cmds[k_unsquished].__module__])
 
 
 def _squish_command_name(cmd):
@@ -240,6 +240,8 @@ class Command(object):
         """Construct an instance of this command."""
         if self.__doc__ == Command.__doc__:
             warn("No help message set for %r" % self)
+        # List of standard options directly supported
+        self.supported_std_options = []
 
     def _maybe_expand_globs(self, file_list):
         """Glob expand file_list if the platform does not do that itself.
@@ -419,12 +421,14 @@ class Command(object):
         """Return dict of valid options for this command.
 
         Maps from long option name to option object."""
-        r = dict()
-        r['help'] = option._help_option
+        r = Option.STD_OPTIONS.copy()
+        std_names = r.keys()
         for o in self.takes_options:
             if isinstance(o, basestring):
                 o = option.Option.OPTIONS[o]
             r[o.name] = o
+            if o.name in std_names:
+                self.supported_std_options.append(o.name)
         return r
 
     def _setup_outf(self):
@@ -445,9 +449,8 @@ class Command(object):
 
         output_encoding = osutils.get_terminal_encoding()
 
-        # use 'replace' so that we don't abort if trying to write out
-        # in e.g. the default C locale.
-        self.outf = codecs.getwriter(output_encoding)(sys.stdout, errors=self.encoding_type)
+        self.outf = codecs.getwriter(output_encoding)(sys.stdout,
+                        errors=self.encoding_type)
         # For whatever reason codecs.getwriter() does not advertise its encoding
         # it just returns the encoding of the wrapped file, which is completely
         # bogus. So set the attribute, so we can find the correct encoding later.
@@ -460,9 +463,21 @@ class Command(object):
                  DeprecationWarning, stacklevel=2)
             argv = []
         args, opts = parse_args(self, argv, alias_argv)
+
+        # Process the standard options
         if 'help' in opts:  # e.g. bzr add --help
             sys.stdout.write(self.get_help_text())
             return 0
+        trace.set_verbosity_level(option._verbosity_level)
+        if 'verbose' in self.supported_std_options:
+            opts['verbose'] = trace.is_verbose()
+        elif opts.has_key('verbose'):
+            del opts['verbose']
+        if 'quiet' in self.supported_std_options:
+            opts['quiet'] = trace.is_quiet()
+        elif opts.has_key('quiet'):
+            del opts['quiet']
+
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -511,40 +526,6 @@ class Command(object):
             return None
 
 
-# Technically, this function hasn't been use in a *really* long time
-# but we are only deprecating it now.
-@deprecated_function(zero_eleven)
-def parse_spec(spec):
-    """
-    >>> parse_spec(None)
-    [None, None]
-    >>> parse_spec("./")
-    ['./', None]
-    >>> parse_spec("../@")
-    ['..', -1]
-    >>> parse_spec("../f/@35")
-    ['../f', 35]
-    >>> parse_spec('./@revid:john@arbash-meinel.com-20050711044610-3ca0327c6a222f67')
-    ['.', 'revid:john@arbash-meinel.com-20050711044610-3ca0327c6a222f67']
-    """
-    if spec is None:
-        return [None, None]
-    if '/@' in spec:
-        parsed = spec.split('/@')
-        assert len(parsed) == 2
-        if parsed[1] == "":
-            parsed[1] = -1
-        else:
-            try:
-                parsed[1] = int(parsed[1])
-            except ValueError:
-                pass # We can allow stuff like ./@revid:blahblahblah
-            else:
-                assert parsed[1] >=0
-    else:
-        parsed = [spec, None]
-    return parsed
-
 def parse_args(command, argv, alias_argv=None):
     """Parse command line.
     
@@ -609,6 +590,20 @@ def _match_argform(cmd, takes_args, args):
 
     return argdict
 
+def apply_coveraged(dirname, the_callable, *args, **kwargs):
+    # Cannot use "import trace", as that would import bzrlib.trace instead of
+    # the standard library's trace.
+    trace = __import__('trace')
+
+    tracer = trace.Trace(count=1, trace=0)
+    sys.settrace(tracer.globaltrace)
+
+    ret = the_callable(*args, **kwargs)
+
+    sys.settrace(None)
+    results = tracer.results()
+    results.write_results(show_missing=1, summary=False,
+                          coverdir=dirname)
 
 
 def apply_profiled(the_callable, *args, **kwargs):
@@ -646,6 +641,11 @@ def apply_lsprofiled(filename, the_callable, *args, **kwargs):
     return ret
 
 
+def shlex_split_unicode(unsplit):
+    import shlex
+    return [u.decode('utf-8') for u in shlex.split(unsplit.encode('utf-8'))]
+
+
 def get_alias(cmd, config=None):
     """Return an expanded alias, or None if no alias exists.
 
@@ -661,8 +661,7 @@ def get_alias(cmd, config=None):
         config = bzrlib.config.GlobalConfig()
     alias = config.get_alias(cmd)
     if (alias):
-        import shlex
-        return [a.decode('utf-8') for a in shlex.split(alias.encode('utf-8'))]
+        return shlex_split_unicode(alias)
     return None
 
 
@@ -697,13 +696,16 @@ def run_bzr(argv):
 
     --lsprof
         Run under the Python lsprof profiler.
+
+    --coverage
+        Generate line coverage report in the specified directory.
     """
     argv = list(argv)
     trace.mutter("bzr arguments: %r", argv)
 
     opt_lsprof = opt_profile = opt_no_plugins = opt_builtin =  \
                 opt_no_aliases = False
-    opt_lsprof_file = None
+    opt_lsprof_file = opt_coverage_dir = None
 
     # --no-plugins is handled specially at a very early stage. We need
     # to load plugins before doing other command parsing so that they
@@ -727,8 +729,9 @@ def run_bzr(argv):
             opt_no_aliases = True
         elif a == '--builtin':
             opt_builtin = True
-        elif a in ('--quiet', '-q'):
-            trace.be_quiet()
+        elif a == '--coverage':
+            opt_coverage_dir = argv[i + 1]
+            i += 1
         elif a.startswith('-D'):
             debug.debug_flags.add(a[2:])
         else:
@@ -742,8 +745,8 @@ def run_bzr(argv):
         return 0
 
     if argv[0] == '--version':
-        from bzrlib.version import show_version
-        show_version()
+        from bzrlib.builtins import cmd_version
+        cmd_version().run_argv_aliases([])
         return 0
         
     if not opt_no_plugins:
@@ -772,15 +775,23 @@ def run_bzr(argv):
 
     try:
         if opt_lsprof:
+            if opt_coverage_dir:
+                trace.warning(
+                    '--coverage ignored, because --lsprof is in use.')
             ret = apply_lsprofiled(opt_lsprof_file, run, *run_argv)
         elif opt_profile:
+            if opt_coverage_dir:
+                trace.warning(
+                    '--coverage ignored, because --profile is in use.')
             ret = apply_profiled(run, *run_argv)
+        elif opt_coverage_dir:
+            ret = apply_coveraged(opt_coverage_dir, run, *run_argv)
         else:
             ret = run(*run_argv)
         return ret or 0
     finally:
         # reset, in case we may do other commands later within the same process
-        trace.be_quiet(False)
+        option._verbosity_level = 0
 
 def display_command(func):
     """Decorator that suppresses pipe/interrupt errors."""
@@ -806,24 +817,47 @@ def main(argv):
     import bzrlib.ui
     from bzrlib.ui.text import TextUIFactory
     bzrlib.ui.ui_factory = TextUIFactory()
-    argv = [a.decode(bzrlib.user_encoding) for a in argv[1:]]
+    try:
+        argv = [a.decode(bzrlib.user_encoding) for a in argv[1:]]
+    except UnicodeDecodeError:
+        raise errors.BzrError(("Parameter '%r' is unsupported by the current "
+                                                            "encoding." % a))
     ret = run_bzr_catch_errors(argv)
     trace.mutter("return code %d", ret)
     return ret
 
 
 def run_bzr_catch_errors(argv):
+    # Note: The except clause logic below should be kept in sync with the
+    # profile() routine in lsprof.py.
     try:
         return run_bzr(argv)
     except (KeyboardInterrupt, Exception), e:
         # used to handle AssertionError and KeyboardInterrupt
         # specially here, but hopefully they're handled ok by the logger now
-        trace.report_exception(sys.exc_info(), sys.stderr)
+        exitcode = trace.report_exception(sys.exc_info(), sys.stderr)
         if os.environ.get('BZR_PDB'):
             print '**** entering debugger'
             import pdb
             pdb.post_mortem(sys.exc_traceback)
-        return 3
+        return exitcode
+
+
+def run_bzr_catch_user_errors(argv):
+    """Run bzr and report user errors, but let internal errors propagate.
+
+    This is used for the test suite, and might be useful for other programs
+    that want to wrap the commandline interface.
+    """
+    try:
+        return run_bzr(argv)
+    except Exception, e:
+        if (isinstance(e, (OSError, IOError))
+            or not getattr(e, 'internal_error', True)):
+            trace.report_exception(sys.exc_info(), sys.stderr)
+            return 3
+        else:
+            raise
 
 
 class HelpCommandIndex(object):
