@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,6 +44,12 @@ from bzrlib.patches import (
 # 'unprintable'.
 
 
+# return codes from the bzr program
+EXIT_OK = 0
+EXIT_ERROR = 3
+EXIT_INTERNAL_ERROR = 4
+
+
 class BzrError(StandardError):
     """
     Base class for errors raised by bzrlib.
@@ -95,7 +101,11 @@ class BzrError(StandardError):
         try:
             fmt = self._get_format_string()
             if fmt:
-                s = fmt % self.__dict__
+                d = dict(self.__dict__)
+                # special case: python2.5 puts the 'message' attribute in a
+                # slot, so it isn't seen in __dict__
+                d['message'] = getattr(self, 'message', 'no message')
+                s = fmt % d
                 # __str__() should always return a 'str' object
                 # never a 'unicode' object.
                 if isinstance(s, unicode):
@@ -126,6 +136,17 @@ class BzrError(StandardError):
                )
 
 
+class InternalBzrError(BzrError):
+    """Base class for errors that are internal in nature.
+
+    This is a convenience class for errors that are internal. The
+    internal_error attribute can still be altered in subclasses, if needed.
+    Using this class is simply an easy way to get internal errors.
+    """
+
+    internal_error = True
+
+
 class BzrNewError(BzrError):
     """Deprecated error base class."""
     # base classes should override the docstring with their human-
@@ -164,23 +185,19 @@ class AlreadyBuilding(BzrError):
     _fmt = "The tree builder is already building a tree."
 
 
-class BzrCheckError(BzrError):
+class BzrCheckError(InternalBzrError):
     
     _fmt = "Internal check failed: %(message)s"
 
-    internal_error = True
-
     def __init__(self, message):
         BzrError.__init__(self)
         self.message = message
 
 
-class DisabledMethod(BzrError):
+class DisabledMethod(InternalBzrError):
 
     _fmt = "The smart server method '%(class_name)s' is disabled."
 
-    internal_error = True
-
     def __init__(self, class_name):
         BzrError.__init__(self)
         self.class_name = class_name
@@ -207,12 +224,10 @@ class InProcessTransport(BzrError):
         self.transport = transport
 
 
-class InvalidEntryName(BzrError):
+class InvalidEntryName(InternalBzrError):
     
     _fmt = "Invalid entry name: %(name)s"
 
-    internal_error = True
-
     def __init__(self, name):
         BzrError.__init__(self)
         self.name = name
@@ -237,6 +252,7 @@ class InvalidRevisionId(BzrError):
         self.revision_id = revision_id
         self.branch = branch
 
+
 class ReservedId(BzrError):
 
     _fmt = "Reserved revision-id {%(revision_id)s}"
@@ -245,6 +261,22 @@ class ReservedId(BzrError):
         self.revision_id = revision_id
 
 
+class RootMissing(InternalBzrError):
+
+    _fmt = ("The root entry of a tree must be the first entry supplied to "
+        "record_entry_contents.")
+
+
+class NoPublicBranch(BzrError):
+
+    _fmt = 'There is no public branch set for "%(branch_url)s".'
+
+    def __init__(self, branch):
+        import bzrlib.urlutils as urlutils
+        public_location = urlutils.unescape_for_display(branch.base, 'ascii')
+        BzrError.__init__(self, branch_url=public_location)
+
+
 class NoHelpTopic(BzrError):
 
     _fmt = ("No help could be found for '%(topic)s'. "
@@ -256,7 +288,7 @@ class NoHelpTopic(BzrError):
 
 class NoSuchId(BzrError):
 
-    _fmt = "The file id %(file_id)s is not present in the tree %(tree)s."
+    _fmt = 'The file id "%(file_id)s" is not present in the tree %(tree)s.'
     
     def __init__(self, tree, file_id):
         BzrError.__init__(self)
@@ -264,20 +296,27 @@ class NoSuchId(BzrError):
         self.tree = tree
 
 
-class InventoryModified(BzrError):
+class NoSuchIdInRepository(NoSuchId):
+
+    _fmt = ('The file id "%(file_id)s" is not present in the repository'
+            ' %(repository)r')
+
+    def __init__(self, repository, file_id):
+        BzrError.__init__(self, repository=repository, file_id=file_id)
+
+
+class InventoryModified(InternalBzrError):
 
     _fmt = ("The current inventory for the tree %(tree)r has been modified,"
             " so a clean inventory cannot be read without data loss.")
 
-    internal_error = True
-
     def __init__(self, tree):
         self.tree = tree
 
 
 class NoWorkingTree(BzrError):
 
-    _fmt = "No WorkingTree exists for %(base)s."
+    _fmt = 'No WorkingTree exists for "%(base)s".'
     
     def __init__(self, base):
         BzrError.__init__(self)
@@ -297,20 +336,17 @@ class NotLocalUrl(BzrError):
         self.url = url
 
 
-class WorkingTreeAlreadyPopulated(BzrError):
+class WorkingTreeAlreadyPopulated(InternalBzrError):
 
-    _fmt = """Working tree already populated in %(base)s"""
-
-    internal_error = True
+    _fmt = 'Working tree already populated in "%(base)s"'
 
     def __init__(self, base):
         self.base = base
 
+
 class BzrCommandError(BzrError):
     """Error from user command"""
 
-    internal_error = False
-
     # Error from malformed user command; please avoid raising this as a
     # generic exception not caused by user input.
     #
@@ -445,7 +481,8 @@ class RenameFailedFilesExist(BzrError):
     """Used when renaming and both source and dest exist."""
 
     _fmt = ("Could not rename %(source)s => %(dest)s because both files exist."
-            "%(extra)s")
+            " (Use --after to tell bzr about a rename that has already"
+            " happened)%(extra)s")
 
     def __init__(self, source, dest, extra=None):
         BzrError.__init__(self)
@@ -459,49 +496,52 @@ class RenameFailedFilesExist(BzrError):
 
 class NotADirectory(PathError):
 
-    _fmt = "%(path)r is not a directory %(extra)s"
+    _fmt = '"%(path)s" is not a directory %(extra)s'
 
 
 class NotInWorkingDirectory(PathError):
 
-    _fmt = "%(path)r is not in the working directory %(extra)s"
+    _fmt = '"%(path)s" is not in the working directory %(extra)s'
 
 
 class DirectoryNotEmpty(PathError):
 
-    _fmt = "Directory not empty: %(path)r%(extra)s"
+    _fmt = 'Directory not empty: "%(path)s"%(extra)s'
 
 
-class ReadingCompleted(BzrError):
+class HardLinkNotSupported(PathError):
+
+    _fmt = 'Hard-linking "%(path)s" is not supported'
+
+
+class ReadingCompleted(InternalBzrError):
     
     _fmt = ("The MediumRequest '%(request)s' has already had finish_reading "
             "called upon it - the request has been completed and no more "
             "data may be read.")
 
-    internal_error = True
-
     def __init__(self, request):
         self.request = request
 
 
 class ResourceBusy(PathError):
 
-    _fmt = "Device or resource busy: %(path)r%(extra)s"
+    _fmt = 'Device or resource busy: "%(path)s"%(extra)s'
 
 
 class PermissionDenied(PathError):
 
-    _fmt = "Permission denied: %(path)r%(extra)s"
+    _fmt = 'Permission denied: "%(path)s"%(extra)s'
 
 
 class InvalidURL(PathError):
 
-    _fmt = "Invalid url supplied to transport: %(path)r%(extra)s"
+    _fmt = 'Invalid url supplied to transport: "%(path)s"%(extra)s'
 
 
 class InvalidURLJoin(PathError):
 
-    _fmt = "Invalid URL join request: %(args)s%(extra)s"
+    _fmt = 'Invalid URL join request: "%(args)s"%(extra)s'
 
     def __init__(self, msg, base, args):
         PathError.__init__(self, base, msg)
@@ -533,8 +573,8 @@ class ReadError(PathError):
 
 class ShortReadvError(PathError):
 
-    _fmt = ("readv() read %(actual)s bytes rather than %(length)s bytes"
-            " at %(offset)s for %(path)s%(extra)s")
+    _fmt = ('readv() read %(actual)s bytes rather than %(length)s bytes'
+            ' at %(offset)s for "%(path)s"%(extra)s')
 
     internal_error = True
 
@@ -547,7 +587,7 @@ class ShortReadvError(PathError):
 
 class PathNotChild(PathError):
 
-    _fmt = "Path %(path)r is not a child of path %(base)r%(extra)s"
+    _fmt = 'Path "%(path)s" is not a child of path "%(base)s"%(extra)s'
 
     internal_error = True
 
@@ -563,7 +603,7 @@ class PathNotChild(PathError):
 
 class InvalidNormalization(PathError):
 
-    _fmt = "Path %(path)r is not unicode normalized"
+    _fmt = 'Path "%(path)s" is not unicode normalized'
 
 
 # TODO: This is given a URL; we try to unescape it but doing that from inside
@@ -571,7 +611,7 @@ class InvalidNormalization(PathError):
 # TODO: Probably this behavior of should be a common superclass 
 class NotBranchError(PathError):
 
-    _fmt = "Not a branch: %(path)s"
+    _fmt = 'Not a branch: "%(path)s".'
 
     def __init__(self, path):
        import bzrlib.urlutils as urlutils
@@ -589,19 +629,19 @@ class NoSubmitBranch(PathError):
 
 class AlreadyBranchError(PathError):
 
-    _fmt = "Already a branch: %(path)s."
+    _fmt = 'Already a branch: "%(path)s".'
 
 
 class BranchExistsWithoutWorkingTree(PathError):
 
-    _fmt = "Directory contains a branch, but no working tree \
-(use bzr checkout if you wish to build a working tree): %(path)s"
+    _fmt = 'Directory contains a branch, but no working tree \
+(use bzr checkout if you wish to build a working tree): "%(path)s"'
 
 
 class AtomicFileAlreadyClosed(PathError):
 
-    _fmt = ("'%(function)s' called on an AtomicFile after it was closed:"
-            " %(path)s")
+    _fmt = ('"%(function)s" called on an AtomicFile after it was closed:'
+            ' "%(path)s"')
 
     def __init__(self, path, function):
         PathError.__init__(self, path=path, extra=None)
@@ -610,8 +650,8 @@ class AtomicFileAlreadyClosed(PathError):
 
 class InaccessibleParent(PathError):
 
-    _fmt = ("Parent not accessible given base %(base)s and"
-            " relative path %(path)s")
+    _fmt = ('Parent not accessible given base "%(base)s" and'
+            ' relative path "%(path)s"')
 
     def __init__(self, path, base):
         PathError.__init__(self, path)
@@ -620,7 +660,7 @@ class InaccessibleParent(PathError):
 
 class NoRepositoryPresent(BzrError):
 
-    _fmt = "No repository present: %(path)r"
+    _fmt = 'No repository present: "%(path)s"'
     def __init__(self, bzrdir):
         BzrError.__init__(self)
         self.path = bzrdir.transport.clone('..').base
@@ -628,7 +668,7 @@ class NoRepositoryPresent(BzrError):
 
 class FileInWrongBranch(BzrError):
 
-    _fmt = "File %(path)s in not in branch %(branch_base)s."
+    _fmt = 'File "%(path)s" is not in branch %(branch_base)s.'
 
     def __init__(self, branch, path):
         BzrError.__init__(self)
@@ -644,7 +684,11 @@ class UnsupportedFormatError(BzrError):
 
 class UnknownFormatError(BzrError):
     
-    _fmt = "Unknown branch format: %(format)r"
+    _fmt = "Unknown %(kind)s format: %(format)r"
+
+    def __init__(self, format, kind='branch'):
+        self.kind = kind
+        self.format = format
 
 
 class IncompatibleFormat(BzrError):
@@ -678,7 +722,7 @@ class IncompatibleRevision(BzrError):
 class AlreadyVersionedError(BzrError):
     """Used when a path is expected not to be versioned, but it is."""
 
-    _fmt = "%(context_info)s%(path)s is already versioned"
+    _fmt = "%(context_info)s%(path)s is already versioned."
 
     def __init__(self, path, context_info=None):
         """Construct a new AlreadyVersionedError.
@@ -699,7 +743,7 @@ class AlreadyVersionedError(BzrError):
 class NotVersionedError(BzrError):
     """Used when a path is expected to be versioned, but it is not."""
 
-    _fmt = "%(context_info)s%(path)s is not versioned"
+    _fmt = "%(context_info)s%(path)s is not versioned."
 
     def __init__(self, path, context_info=None):
         """Construct a new NotVersionedError.
@@ -758,15 +802,13 @@ class BadFileKindError(BzrError):
 
 class ForbiddenControlFileError(BzrError):
 
-    _fmt = "Cannot operate on %(filename)s because it is a control file"
+    _fmt = 'Cannot operate on "%(filename)s" because it is a control file'
 
 
-class LockError(BzrError):
+class LockError(InternalBzrError):
 
     _fmt = "Lock error: %(msg)s"
 
-    internal_error = True
-
     # All exceptions from the lock/unlock functions should be from
     # this exception class.  They will be translated as necessary. The
     # original exception is available as e.original_error
@@ -774,8 +816,8 @@ class LockError(BzrError):
     # New code should prefer to raise specific subclasses
     def __init__(self, message):
         # Python 2.5 uses a slot for StandardError.message,
-        # so use a different variable name
-        # so it is exposed in self.__dict__
+        # so use a different variable name.  We now work around this in
+        # BzrError.__str__, but this member name is kept for compatability.
         self.msg = message
 
 
@@ -820,12 +862,25 @@ class ReadOnlyLockError(LockError):
 
     _fmt = "Cannot acquire write lock on %(fname)s. %(msg)s"
 
+    @symbol_versioning.deprecated_method(symbol_versioning.zero_ninetytwo)
     def __init__(self, fname, msg):
         LockError.__init__(self, '')
         self.fname = fname
         self.msg = msg
 
 
+class LockFailed(LockError):
+
+    internal_error = False
+
+    _fmt = "Cannot lock %(lock)s: %(why)s"
+
+    def __init__(self, lock, why):
+        LockError.__init__(self, '')
+        self.lock = lock
+        self.why = why
+
+
 class OutSideTransaction(BzrError):
 
     _fmt = ("A transaction related operation was attempted after"
@@ -853,6 +908,8 @@ class ReadOnlyObjectDirtiedError(ReadOnlyError):
 
 class UnlockableTransport(LockError):
 
+    internal_error = False
+
     _fmt = "Cannot lock: transport is read only: %(transport)s"
 
     def __init__(self, transport):
@@ -861,7 +918,7 @@ class UnlockableTransport(LockError):
 
 class LockContention(LockError):
 
-    _fmt = "Could not acquire lock %(lock)s"
+    _fmt = 'Could not acquire lock "%(lock)s"'
     # TODO: show full url for lock, combining the transport and relative
     # bits?
 
@@ -909,8 +966,6 @@ class TokenLockingNotSupported(LockError):
 
     _fmt = "The object %(obj)s does not support token specifying a token when locking."
 
-    internal_error = True
-
     def __init__(self, obj):
         self.obj = obj
 
@@ -941,6 +996,12 @@ class CannotCommitSelectedFileMerge(BzrError):
         BzrError.__init__(self, files=files, files_str=files_str)
 
 
+class BadCommitMessageEncoding(BzrError):
+
+    _fmt = 'The specified commit message contains characters unsupported by '\
+        'the current encoding.'
+
+
 class UpgradeReadonly(BzrError):
 
     _fmt = "Upgrade URL cannot work with readonly URLs."
@@ -960,29 +1021,32 @@ class StrictCommitFailed(Exception):
     _fmt = "Commit refused because there are unknowns in the tree."
 
 
-class NoSuchRevision(BzrError):
+class NoSuchRevision(InternalBzrError):
 
-    _fmt = "Branch %(branch)s has no revision %(revision)s"
-
-    internal_error = True
+    _fmt = "%(branch)s has no revision %(revision)s"
 
     def __init__(self, branch, revision):
+        # 'branch' may sometimes be an internal object like a KnitRevisionStore
         BzrError.__init__(self, branch=branch, revision=revision)
 
 
-class NotLeftParentDescendant(BzrError):
+# zero_ninetyone: this exception is no longer raised and should be removed
+class NotLeftParentDescendant(InternalBzrError):
 
     _fmt = ("Revision %(old_revision)s is not the left parent of"
             " %(new_revision)s, but branch %(branch_location)s expects this")
 
-    internal_error = True
-
     def __init__(self, branch, old_revision, new_revision):
         BzrError.__init__(self, branch_location=branch.base,
                           old_revision=old_revision,
                           new_revision=new_revision)
 
 
+class RangeInChangeOption(BzrError):
+
+    _fmt = "Option --change does not accept revision ranges"
+
+
 class NoSuchRevisionSpec(BzrError):
 
     _fmt = "No namespace registered for string: %(spec)r"
@@ -994,7 +1058,7 @@ class NoSuchRevisionSpec(BzrError):
 class NoSuchRevisionInTree(NoSuchRevision):
     """When using Tree.revision_tree, and the revision is not accessible."""
     
-    _fmt = "The revision id %(revision_id)s is not present in the tree %(tree)s."
+    _fmt = "The revision id {%(revision_id)s} is not present in the tree %(tree)s."
 
     def __init__(self, tree, revision_id):
         BzrError.__init__(self)
@@ -1037,19 +1101,15 @@ class DivergedBranches(BzrError):
     _fmt = ("These branches have diverged."
             " Use the merge command to reconcile them.")
 
-    internal_error = False
-
     def __init__(self, branch1, branch2):
         self.branch1 = branch1
         self.branch2 = branch2
 
 
-class NotLefthandHistory(BzrError):
+class NotLefthandHistory(InternalBzrError):
 
     _fmt = "Supplied history does not follow left-hand parents"
 
-    internal_error = True
-
     def __init__(self, history):
         BzrError.__init__(self, history=history)
 
@@ -1059,7 +1119,11 @@ class UnrelatedBranches(BzrError):
     _fmt = ("Branches have no common ancestor, and"
             " no merge base revision was specified.")
 
-    internal_error = False
+
+class CannotReverseCherrypick(BzrError):
+
+    _fmt = ('Selected merge cannot perform reverse cherrypicks.  Try merge3'
+            ' or diff3.')
 
 
 class NoCommonAncestor(BzrError):
@@ -1132,8 +1196,8 @@ class UnlistableBranch(BzrError):
 
 class BoundBranchOutOfDate(BzrError):
 
-    _fmt = ("Bound branch %(branch)s is out of date"
-            " with master branch %(master)s.")
+    _fmt = ("Bound branch %(branch)s is out of date with master branch"
+            " %(master)s.")
 
     def __init__(self, branch, master):
         BzrError.__init__(self)
@@ -1215,7 +1279,7 @@ class WeaveFormatError(WeaveError):
 
 class WeaveParentMismatch(WeaveError):
 
-    _fmt = "Parents are mismatched between two revisions."
+    _fmt = "Parents are mismatched between two revisions. %(message)s"
     
 
 class WeaveInvalidChecksum(WeaveError):
@@ -1254,7 +1318,7 @@ class VersionedFileError(BzrError):
 
 class RevisionNotPresent(VersionedFileError):
     
-    _fmt = "Revision {%(revision_id)s} not present in %(file_id)s."
+    _fmt = 'Revision {%(revision_id)s} not present in "%(file_id)s".'
 
     def __init__(self, revision_id, file_id):
         VersionedFileError.__init__(self)
@@ -1264,7 +1328,7 @@ class RevisionNotPresent(VersionedFileError):
 
 class RevisionAlreadyPresent(VersionedFileError):
     
-    _fmt = "Revision {%(revision_id)s} already present in %(file_id)s."
+    _fmt = 'Revision {%(revision_id)s} already present in "%(file_id)s".'
 
     def __init__(self, revision_id, file_id):
         VersionedFileError.__init__(self)
@@ -1277,22 +1341,10 @@ class VersionedFileInvalidChecksum(VersionedFileError):
     _fmt = "Text did not match its checksum: %(message)s"
 
 
-class KnitError(BzrError):
+class KnitError(InternalBzrError):
     
     _fmt = "Knit error"
 
-    internal_error = True
-
-
-class KnitHeaderError(KnitError):
-
-    _fmt = "Knit header error: %(badline)r unexpected for file %(filename)s"
-
-    def __init__(self, badline, filename):
-        KnitError.__init__(self)
-        self.badline = badline
-        self.filename = filename
-
 
 class KnitCorrupt(KnitError):
 
@@ -1304,6 +1356,35 @@ class KnitCorrupt(KnitError):
         self.how = how
 
 
+class KnitDataStreamIncompatible(KnitError):
+    # Not raised anymore, as we can convert data streams.  In future we may
+    # need it again for more exotic cases, so we're keeping it around for now.
+
+    _fmt = "Cannot insert knit data stream of format \"%(stream_format)s\" into knit of format \"%(target_format)s\"."
+
+    def __init__(self, stream_format, target_format):
+        self.stream_format = stream_format
+        self.target_format = target_format
+        
+
+class KnitDataStreamUnknown(KnitError):
+    # Indicates a data stream we don't know how to handle.
+
+    _fmt = "Cannot parse knit data stream of format \"%(stream_format)s\"."
+
+    def __init__(self, stream_format):
+        self.stream_format = stream_format
+        
+
+class KnitHeaderError(KnitError):
+
+    _fmt = 'Knit header error: %(badline)r unexpected for file "%(filename)s".'
+
+    def __init__(self, badline, filename):
+        KnitError.__init__(self)
+        self.badline = badline
+        self.filename = filename
+
 class KnitIndexUnknownMethod(KnitError):
     """Raised when we don't understand the storage method.
 
@@ -1344,14 +1425,12 @@ class TransportError(BzrError):
         BzrError.__init__(self)
 
 
-class TooManyConcurrentRequests(BzrError):
+class TooManyConcurrentRequests(InternalBzrError):
 
     _fmt = ("The medium '%(medium)s' has reached its concurrent request limit."
             " Be sure to finish_writing and finish_reading on the"
             " currently open request.")
 
-    internal_error = True
-
     def __init__(self, medium):
         self.medium = medium
 
@@ -1401,11 +1480,10 @@ class ConnectionReset(TransportError):
 
 class InvalidRange(TransportError):
 
-    _fmt = "Invalid range access in %(path)s at %(offset)s."
-    
-    def __init__(self, path, offset):
-        TransportError.__init__(self, ("Invalid range access in %s at %d"
-                                       % (path, offset)))
+    _fmt = "Invalid range access in %(path)s at %(offset)s: %(msg)s"
+
+    def __init__(self, path, offset, msg=None):
+        TransportError.__init__(self, msg)
         self.path = path
         self.offset = offset
 
@@ -1422,7 +1500,7 @@ class InvalidHttpResponse(TransportError):
 class InvalidHttpRange(InvalidHttpResponse):
 
     _fmt = "Invalid http range %(range)r for %(path)s: %(msg)s"
-    
+
     def __init__(self, path, range, msg):
         self.range = range
         InvalidHttpResponse.__init__(self, path, msg)
@@ -1431,7 +1509,7 @@ class InvalidHttpRange(InvalidHttpResponse):
 class InvalidHttpContentType(InvalidHttpResponse):
 
     _fmt = 'Invalid http Content-type "%(ctype)s" for %(path)s: %(msg)s'
-    
+
     def __init__(self, path, ctype, msg):
         self.ctype = ctype
         InvalidHttpResponse.__init__(self, path, msg)
@@ -1441,14 +1519,13 @@ class RedirectRequested(TransportError):
 
     _fmt = '%(source)s is%(permanently)s redirected to %(target)s'
 
-    def __init__(self, source, target, is_permament=False, qual_proto=None):
+    def __init__(self, source, target, is_permanent=False, qual_proto=None):
         self.source = source
         self.target = target
-        if is_permament:
+        if is_permanent:
             self.permanently = ' permanently'
         else:
             self.permanently = ''
-        self.is_permament = is_permament
         self._qualified_proto = qual_proto
         TransportError.__init__(self)
 
@@ -1490,6 +1567,7 @@ class TooManyRedirections(TransportError):
 
     _fmt = "Too many redirections"
 
+
 class ConflictsInTree(BzrError):
 
     _fmt = "Working tree has conflicts."
@@ -1516,7 +1594,7 @@ class NoEmailInUsername(BzrError):
 
 class SigningFailed(BzrError):
 
-    _fmt = "Failed to gpg sign data with command %(command_line)r"
+    _fmt = 'Failed to gpg sign data with command "%(command_line)s"'
 
     def __init__(self, command_line):
         BzrError.__init__(self, command_line=command_line)
@@ -1547,25 +1625,21 @@ class GraphCycleError(BzrError):
         self.graph = graph
 
 
-class WritingCompleted(BzrError):
+class WritingCompleted(InternalBzrError):
 
     _fmt = ("The MediumRequest '%(request)s' has already had finish_writing "
             "called upon it - accept bytes may not be called anymore.")
 
-    internal_error = True
-
     def __init__(self, request):
         self.request = request
 
 
-class WritingNotComplete(BzrError):
+class WritingNotComplete(InternalBzrError):
 
     _fmt = ("The MediumRequest '%(request)s' has not has finish_writing "
             "called upon it - until the write phase is complete no "
             "data may be read.")
 
-    internal_error = True
-
     def __init__(self, request):
         self.request = request
 
@@ -1579,12 +1653,10 @@ class NotConflicted(BzrError):
         self.filename = filename
 
 
-class MediumNotConnected(BzrError):
+class MediumNotConnected(InternalBzrError):
 
     _fmt = """The medium '%(medium)s' is not connected."""
 
-    internal_error = True
-
     def __init__(self, medium):
         self.medium = medium
 
@@ -1596,7 +1668,7 @@ class MustUseDecorated(Exception):
 
 class NoBundleFound(BzrError):
 
-    _fmt = "No bundle was found in %(filename)s"
+    _fmt = 'No bundle was found in "%(filename)s".'
 
     def __init__(self, filename):
         BzrError.__init__(self)
@@ -1625,6 +1697,7 @@ class MissingText(BzrError):
         self.text_revision = text_revision
         self.file_id = file_id
 
+
 class DuplicateFileId(BzrError):
 
     _fmt = "File id {%(file_id)s} already exists in inventory as %(entry)s"
@@ -1665,12 +1738,10 @@ class NoFinalPath(BzrError):
         self.root_trans_id = transform.root
 
 
-class BzrBadParameter(BzrError):
+class BzrBadParameter(InternalBzrError):
 
     _fmt = "Bad parameter: %(param)r"
 
-    internal_error = True
-
     # This exception should never be thrown, but it is a base class for all
     # parameter-to-function errors.
 
@@ -1738,7 +1809,8 @@ class BzrRenameFailedError(BzrMoveFailedError):
 class BzrRemoveChangedFilesError(BzrError):
     """Used when user is trying to remove changed files."""
 
-    _fmt = ("Can't remove changed or unknown files:\n%(changes_as_text)s"
+    _fmt = ("Can't safely remove modified or unknown files:\n"
+        "%(changes_as_text)s"
         "Use --keep to not delete them, or --force to delete them regardless.")
 
     def __init__(self, tree_delta):
@@ -1809,6 +1881,22 @@ class BadConversionTarget(BzrError):
         self.format = format
 
 
+class NoDiffFound(BzrError):
+
+    _fmt = 'Could not find an appropriate Differ for file "%(path)s"'
+
+    def __init__(self, path):
+        BzrError.__init__(self, path)
+
+
+class ExecutableMissing(BzrError):
+
+    _fmt = "%(exe_name)s could not be found on this machine"
+
+    def __init__(self, exe_name):
+        BzrError.__init__(self, exe_name=exe_name)
+
+
 class NoDiff(BzrError):
 
     _fmt = "Diff is not installed on this machine: %(msg)s"
@@ -1822,6 +1910,12 @@ class NoDiff3(BzrError):
     _fmt = "Diff3 is not installed on this machine."
 
 
+class ExistingContent(BzrError):
+    # Added in bzrlib 0.92, used by VersionedFile.add_lines.
+
+    _fmt = "The content being inserted is already present."
+
+
 class ExistingLimbo(BzrError):
 
     _fmt = """This tree contains left-over files from a failed operation.
@@ -1833,9 +1927,19 @@ class ExistingLimbo(BzrError):
        self.limbo_dir = limbo_dir
 
 
+class ExistingPendingDeletion(BzrError):
+
+    _fmt = """This tree contains left-over files from a failed operation.
+    Please examine %(pending_deletion)s to see if it contains any files you
+    wish to keep, and delete it when you are done."""
+
+    def __init__(self, pending_deletion):
+       BzrError.__init__(self, pending_deletion=pending_deletion)
+
+
 class ImmortalLimbo(BzrError):
 
-    _fmt = """Unable to delete transform temporary directory $(limbo_dir)s.
+    _fmt = """Unable to delete transform temporary directory %(limbo_dir)s.
     Please examine %(limbo_dir)s to see if it contains any files you wish to
     keep, and delete it when you are done."""
 
@@ -1844,6 +1948,16 @@ class ImmortalLimbo(BzrError):
        self.limbo_dir = limbo_dir
 
 
+class ImmortalPendingDeletion(BzrError):
+
+    _fmt = ("Unable to delete transform temporary directory "
+    "%(pending_deletion)s.  Please examine %(pending_deletion)s to see if it "
+    "contains any files you wish to keep, and delete it when you are done.")
+
+    def __init__(self, pending_deletion):
+       BzrError.__init__(self, pending_deletion=pending_deletion)
+
+
 class OutOfDateTree(BzrError):
 
     _fmt = "Working tree is out of date, please run 'bzr update'."
@@ -1876,6 +1990,17 @@ class ConflictFormatError(BzrError):
     _fmt = "Format error in conflict listings"
 
 
+class CorruptDirstate(BzrError):
+
+    _fmt = ("Inconsistency in dirstate file %(dirstate_path)s.\n"
+            "Error: %(description)s")
+
+    def __init__(self, dirstate_path, description):
+        BzrError.__init__(self)
+        self.dirstate_path = dirstate_path
+        self.description = description
+
+
 class CorruptRepository(BzrError):
 
     _fmt = ("An error has been detected in the repository %(repo_path)s.\n"
@@ -1886,6 +2011,19 @@ class CorruptRepository(BzrError):
         self.repo_path = repo.bzrdir.root_transport.base
 
 
+class InconsistentDelta(BzrError):
+    """Used when we get a delta that is not valid."""
+
+    _fmt = ("An inconsistent delta was supplied involving %(path)r,"
+            " %(file_id)r\nreason: %(reason)s")
+
+    def __init__(self, path, file_id, reason):
+        BzrError.__init__(self)
+        self.path = path
+        self.file_id = file_id
+        self.reason = reason
+
+
 class UpgradeRequired(BzrError):
 
     _fmt = "To use this feature you must upgrade your branch at %(path)s."
@@ -2032,12 +2170,10 @@ class RootNotRich(BzrError):
     _fmt = """This operation requires rich root data storage"""
 
 
-class NoSmartMedium(BzrError):
+class NoSmartMedium(InternalBzrError):
 
     _fmt = "The transport '%(transport)s' cannot tunnel the smart protocol."
 
-    internal_error = True
-
     def __init__(self, transport):
         self.transport = transport
 
@@ -2074,13 +2210,11 @@ class GhostRevisionUnusableHere(BzrError):
         self.revision_id = revision_id
 
 
-class IllegalUseOfScopeReplacer(BzrError):
+class IllegalUseOfScopeReplacer(InternalBzrError):
 
     _fmt = ("ScopeReplacer object %(name)r was used incorrectly:"
             " %(msg)s%(extra)s")
 
-    internal_error = True
-
     def __init__(self, name, msg, extra=None):
         BzrError.__init__(self)
         self.name = name
@@ -2091,25 +2225,21 @@ class IllegalUseOfScopeReplacer(BzrError):
             self.extra = ''
 
 
-class InvalidImportLine(BzrError):
+class InvalidImportLine(InternalBzrError):
 
     _fmt = "Not a valid import statement: %(msg)\n%(text)s"
 
-    internal_error = True
-
     def __init__(self, text, msg):
         BzrError.__init__(self)
         self.text = text
         self.msg = msg
 
 
-class ImportNameCollision(BzrError):
+class ImportNameCollision(InternalBzrError):
 
     _fmt = ("Tried to import an object to the same name as"
             " an existing object. %(name)s")
 
-    internal_error = True
-
     def __init__(self, name):
         BzrError.__init__(self)
         self.name = name
@@ -2147,7 +2277,7 @@ class PatchVerificationFailed(BzrError):
 class PatchMissing(BzrError):
     """Raise a patch type was specified but no patch supplied"""
 
-    _fmt = "patch_type was %(patch_type)s, but no patch was supplied."
+    _fmt = "Patch_type was %(patch_type)s, but no patch was supplied."
 
     def __init__(self, patch_type):
         BzrError.__init__(self)
@@ -2164,7 +2294,7 @@ class UnsupportedInventoryKind(BzrError):
 
 class BadSubsumeSource(BzrError):
 
-    _fmt = """Can't subsume %(other_tree)s into %(tree)s.  %(reason)s"""
+    _fmt = "Can't subsume %(other_tree)s into %(tree)s. %(reason)s"
 
     def __init__(self, tree, other_tree, reason):
         self.tree = tree
@@ -2180,11 +2310,10 @@ class SubsumeTargetNeedsUpgrade(BzrError):
         self.other_tree = other_tree
 
 
-class BadReferenceTarget(BzrError):
+class BadReferenceTarget(InternalBzrError):
 
-    _fmt = "Can't add reference to %(other_tree)s into %(tree)s.  %(reason)s"
-
-    internal_error = True
+    _fmt = "Can't add reference to %(other_tree)s into %(tree)s." \
+           "%(reason)s"
 
     def __init__(self, tree, other_tree, reason):
         self.tree = tree
@@ -2226,6 +2355,16 @@ class MalformedBugIdentifier(BzrError):
         self.reason = reason
 
 
+class InvalidBugTrackerURL(BzrError):
+
+    _fmt = ("The URL for bug tracker \"%(abbreviation)s\" doesn't "
+            "contain {id}: %(url)s")
+
+    def __init__(self, abbreviation, url):
+        self.abbreviation = abbreviation
+        self.url = url
+
+
 class UnknownBugTrackerAbbreviation(BzrError):
 
     _fmt = ("Cannot find registered bug tracker called %(abbreviation)s "
@@ -2260,8 +2399,6 @@ class UnexpectedEndOfContainerError(ContainerError):
 
     _fmt = "Unexpected end of container stream"
 
-    internal_error = False
-
 
 class UnknownRecordTypeError(ContainerError):
 
@@ -2289,17 +2426,23 @@ class ContainerHasExcessDataError(ContainerError):
 
 class DuplicateRecordNameError(ContainerError):
 
-    _fmt = "Container has multiple records with the same name: \"%(name)s\""
+    _fmt = "Container has multiple records with the same name: %(name)s"
 
     def __init__(self, name):
         self.name = name
 
 
-class NoDestinationAddress(BzrError):
+class NoDestinationAddress(InternalBzrError):
 
     _fmt = "Message does not have a destination address."
 
-    internal_error = True
+
+class RepositoryDataStreamError(BzrError):
+
+    _fmt = "Corrupt or incompatible data stream: %(reason)s"
+
+    def __init__(self, reason):
+        self.reason = reason
 
 
 class SMTPError(BzrError):
@@ -2310,6 +2453,34 @@ class SMTPError(BzrError):
         self.error = error
 
 
+class NoMessageSupplied(BzrError):
+
+    _fmt = "No message supplied."
+
+
+class NoMailAddressSpecified(BzrError):
+
+    _fmt = "No mail-to address specified."
+
+
+class UnknownMailClient(BzrError):
+
+    _fmt = "Unknown mail client: %(mail_client)s"
+
+    def __init__(self, mail_client):
+        BzrError.__init__(self, mail_client=mail_client)
+
+
+class MailClientNotFound(BzrError):
+
+    _fmt = "Unable to find mail client with the following names:"\
+        " %(mail_command_list_string)s"
+
+    def __init__(self, mail_command_list):
+        mail_command_list_string = ', '.join(mail_command_list)
+        BzrError.__init__(self, mail_command_list=mail_command_list,
+                          mail_command_list_string=mail_command_list_string)
+
 class SMTPConnectionRefused(SMTPError):
 
     _fmt = "SMTP connection to %(host)s refused"
@@ -2322,3 +2493,101 @@ class SMTPConnectionRefused(SMTPError):
 class DefaultSMTPConnectionRefused(SMTPConnectionRefused):
 
     _fmt = "Please specify smtp_server.  No server at default %(host)s."
+
+
+class BzrDirError(BzrError):
+
+    def __init__(self, bzrdir):
+        import bzrlib.urlutils as urlutils
+        display_url = urlutils.unescape_for_display(bzrdir.root_transport.base,
+                                                    'ascii')
+        BzrError.__init__(self, bzrdir=bzrdir, display_url=display_url)
+
+
+class AlreadyBranch(BzrDirError):
+
+    _fmt = "'%(display_url)s' is already a branch."
+
+
+class AlreadyTree(BzrDirError):
+
+    _fmt = "'%(display_url)s' is already a tree."
+
+
+class AlreadyCheckout(BzrDirError):
+
+    _fmt = "'%(display_url)s' is already a checkout."
+
+
+class AlreadyLightweightCheckout(BzrDirError):
+
+    _fmt = "'%(display_url)s' is already a lightweight checkout."
+
+
+class ReconfigurationNotSupported(BzrDirError):
+
+    _fmt = "Requested reconfiguration of '%(display_url)s' is not supported."
+
+
+class NoBindLocation(BzrDirError):
+
+    _fmt = "No location could be found to bind to at %(display_url)s."
+
+
+class UncommittedChanges(BzrError):
+
+    _fmt = 'Working tree "%(display_url)s" has uncommitted changes.'
+
+    def __init__(self, tree):
+        import bzrlib.urlutils as urlutils
+        display_url = urlutils.unescape_for_display(
+            tree.bzrdir.root_transport.base, 'ascii')
+        BzrError.__init__(self, tree=tree, display_url=display_url)
+
+
+class MissingTemplateVariable(BzrError):
+
+    _fmt = 'Variable {%(name)s} is not available.'
+
+    def __init__(self, name):
+        self.name = name
+
+
+class NoTemplate(BzrError):
+
+    _fmt = 'No template specified.'
+
+
+class UnableCreateSymlink(BzrError):
+
+    _fmt = 'Unable to create symlink %(path_str)son this platform'
+
+    def __init__(self, path=None):
+        path_str = ''
+        if path:
+            try:
+                path_str = repr(str(path))
+            except UnicodeEncodeError:
+                path_str = repr(path)
+            path_str += ' '
+        self.path_str = path_str
+
+
+class UnsupportedTimezoneFormat(BzrError):
+
+    _fmt = ('Unsupported timezone format "%(timezone)s", '
+            'options are "utc", "original", "local".')
+
+    def __init__(self, timezone):
+        self.timezone = timezone
+
+
+class UnableEncodePath(BzrError):
+
+    _fmt = ('Unable to encode %(kind)s path %(path)r in '
+            'user encoding %(user_encoding)s')
+
+    def __init__(self, path, kind):
+        self.path = path
+        self.kind = kind
+        self.user_encoding = osutils.get_user_encoding()
