@@ -49,22 +49,38 @@ listing other things that were changed in the same revision, but not
 all the changes since the previous revision that touched hello.c.
 """
 
-# TODO: option to show delta summaries for merged-in revisions
-
-from itertools import izip
+import codecs
+from itertools import (
+    izip,
+    )
 import re
+import sys
+from warnings import (
+    warn,
+    )
 
 from bzrlib import (
+    config,
+    lazy_regex,
     registry,
     symbol_versioning,
     )
-import bzrlib.errors as errors
-from bzrlib.revisionspec import(
-    RevisionInfo
+from bzrlib.errors import (
+    BzrCommandError,
+    )
+from bzrlib.osutils import (
+    format_date,
+    get_terminal_encoding,
+    terminal_width,
+    )
+from bzrlib.revision import (
+    NULL_REVISION,
+    )
+from bzrlib.revisionspec import (
+    RevisionInfo,
     )
 from bzrlib.symbol_versioning import (
     deprecated_method,
-    zero_eleven,
     zero_seventeen,
     )
 from bzrlib.trace import mutter
@@ -187,11 +203,6 @@ def _show_log(branch,
              search=None,
              limit=None):
     """Worker function for show_log - see show_log."""
-    from bzrlib.osutils import format_date
-    from bzrlib.errors import BzrCheckError
-    
-    from warnings import warn
-
     if not isinstance(lf, LogFormatter):
         warn("not a LogFormatter instance: %r" % lf)
 
@@ -199,7 +210,6 @@ def _show_log(branch,
         mutter('get log for file_id %r', specific_fileid)
 
     if search is not None:
-        import re
         searchRE = re.compile(search, re.IGNORECASE)
     else:
         searchRE = None
@@ -232,11 +242,23 @@ def _show_log(branch,
     else:
         generate_merge_revisions = getattr(lf, 'supports_merge_revisions', 
                                            False)
+    generate_single_revision = False
+    if ((not generate_merge_revisions)
+        and ((start_rev_id and (start_rev_id not in rev_nos))
+            or (end_rev_id and (end_rev_id not in rev_nos)))):
+        generate_single_revision = ((start_rev_id == end_rev_id)
+            and getattr(lf, 'supports_single_merge_revision', False))
+        if not generate_single_revision:
+            raise BzrCommandError('Selected log formatter only supports '
+                'mainline revisions.')
+        generate_merge_revisions = generate_single_revision
     view_revs_iter = get_view_revisions(mainline_revs, rev_nos, branch,
                           direction, include_merges=generate_merge_revisions)
     view_revisions = _filter_revision_range(list(view_revs_iter),
                                             start_rev_id,
                                             end_rev_id)
+    if view_revisions and generate_single_revision:
+        view_revisions = view_revisions[0:1]
     if specific_fileid:
         view_revisions = _filter_revisions_touching_file_id(branch,
                                                          specific_fileid,
@@ -359,8 +381,10 @@ def _get_mainline_revs(branch, start_revision, end_revision):
             branch.check_real_revno(end_revision)
             end_revno = end_revision
 
+    if ((start_rev_id == NULL_REVISION)
+        or (end_rev_id == NULL_REVISION)):
+        raise BzrCommandError('Logging revision 0 is invalid.')
     if start_revno > end_revno:
-        from bzrlib.errors import BzrCommandError
         raise BzrCommandError("Start revision must be older than "
                               "the end revision.")
 
@@ -567,8 +591,12 @@ class LogFormatter(object):
     - supports_delta must be True if this log formatter supports delta.
         Otherwise the delta attribute may not be populated.
     - supports_merge_revisions must be True if this log formatter supports 
-        merge revisions.  If not, only mainline revisions (those 
-        with merge_depth == 0) will be passed to the formatter.
+        merge revisions.  If not, and if supports_single_merge_revisions is
+        also not True, then only mainline revisions will be passed to the 
+        formatter.
+    - supports_single_merge_revision must be True if this log formatter
+        supports logging only a single merge revision.  This flag is
+        only relevant if supports_merge_revisions is not True.
     - supports_tags must be True if this log formatter supports tags.
         Otherwise the tags attribute may not be populated.
     """
@@ -593,7 +621,16 @@ class LogFormatter(object):
         raise NotImplementedError('not implemented in abstract base')
 
     def short_committer(self, rev):
-        return re.sub('<.*@.*>', '', rev.committer).strip(' ')
+        name, address = config.parse_username(rev.committer)
+        if name:
+            return name
+        return address
+
+    def short_author(self, rev):
+        name, address = config.parse_username(rev.get_apparent_author())
+        if name:
+            return name
+        return address
 
 
 class LongLogFormatter(LogFormatter):
@@ -607,11 +644,6 @@ class LongLogFormatter(LogFormatter):
         lr = LogRevision(rev, revno, 0, delta, tags)
         return self.log_revision(lr)
 
-    @deprecated_method(zero_eleven)
-    def show_merge(self, rev, merge_depth):
-        lr = LogRevision(rev, merge_depth=merge_depth)
-        return self.log_revision(lr)
-
     @deprecated_method(zero_seventeen)
     def show_merge_revno(self, rev, merge_depth, revno, tags=None):
         """Show a merged revision rev, with merge_depth and a revno."""
@@ -620,37 +652,40 @@ class LongLogFormatter(LogFormatter):
 
     def log_revision(self, revision):
         """Log a revision, either merged or not."""
-        from bzrlib.osutils import format_date
-        indent = '    '*revision.merge_depth
+        indent = '    ' * revision.merge_depth
         to_file = self.to_file
-        print >>to_file,  indent+'-' * 60
+        to_file.write(indent + '-' * 60 + '\n')
         if revision.revno is not None:
-            print >>to_file,  indent+'revno:', revision.revno
+            to_file.write(indent + 'revno: %s\n' % (revision.revno,))
         if revision.tags:
-            print >>to_file, indent+'tags: %s' % (', '.join(revision.tags))
+            to_file.write(indent + 'tags: %s\n' % (', '.join(revision.tags)))
         if self.show_ids:
-            print >>to_file, indent+'revision-id:', revision.rev.revision_id
+            to_file.write(indent + 'revision-id: ' + revision.rev.revision_id)
+            to_file.write('\n')
             for parent_id in revision.rev.parent_ids:
-                print >>to_file, indent+'parent:', parent_id
-        print >>to_file, indent+'committer:', revision.rev.committer
+                to_file.write(indent + 'parent: %s\n' % (parent_id,))
+
+        author = revision.rev.properties.get('author', None)
+        if author is not None:
+            to_file.write(indent + 'author: %s\n' % (author,))
+        to_file.write(indent + 'committer: %s\n' % (revision.rev.committer,))
+
+        branch_nick = revision.rev.properties.get('branch-nick', None)
+        if branch_nick is not None:
+            to_file.write(indent + 'branch nick: %s\n' % (branch_nick,))
 
-        try:
-            print >>to_file, indent+'branch nick: %s' % \
-                revision.rev.properties['branch-nick']
-        except KeyError:
-            pass
         date_str = format_date(revision.rev.timestamp,
                                revision.rev.timezone or 0,
                                self.show_timezone)
-        print >>to_file,  indent+'timestamp: %s' % date_str
+        to_file.write(indent + 'timestamp: %s\n' % (date_str,))
 
-        print >>to_file,  indent+'message:'
+        to_file.write(indent + 'message:\n')
         if not revision.rev.message:
-            print >>to_file,  indent+'  (no message)'
+            to_file.write(indent + '  (no message)\n')
         else:
             message = revision.rev.message.rstrip('\r\n')
             for l in message.split('\n'):
-                print >>to_file,  indent+'  ' + l
+                to_file.write(indent + '  %s\n' % (l,))
         if revision.delta is not None:
             revision.delta.show(to_file, self.show_ids, indent=indent)
 
@@ -658,6 +693,7 @@ class LongLogFormatter(LogFormatter):
 class ShortLogFormatter(LogFormatter):
 
     supports_delta = True
+    supports_single_merge_revision = True
 
     @deprecated_method(zero_seventeen)
     def show(self, revno, rev, delta):
@@ -665,8 +701,6 @@ class ShortLogFormatter(LogFormatter):
         return self.log_revision(lr)
 
     def log_revision(self, revision):
-        from bzrlib.osutils import format_date
-
         to_file = self.to_file
         date_str = format_date(revision.rev.timestamp,
                                revision.rev.timezone or 0,
@@ -674,33 +708,34 @@ class ShortLogFormatter(LogFormatter):
         is_merge = ''
         if len(revision.rev.parent_ids) > 1:
             is_merge = ' [merge]'
-        print >>to_file, "%5s %s\t%s%s" % (revision.revno,
-                self.short_committer(revision.rev),
+        to_file.write("%5s %s\t%s%s\n" % (revision.revno,
+                self.short_author(revision.rev),
                 format_date(revision.rev.timestamp,
                             revision.rev.timezone or 0,
                             self.show_timezone, date_fmt="%Y-%m-%d",
                             show_offset=False),
-                is_merge)
+                is_merge))
         if self.show_ids:
-            print >>to_file,  '      revision-id:', revision.rev.revision_id
+            to_file.write('      revision-id:%s\n' % (revision.rev.revision_id,))
         if not revision.rev.message:
-            print >>to_file,  '      (no message)'
+            to_file.write('      (no message)\n')
         else:
             message = revision.rev.message.rstrip('\r\n')
             for l in message.split('\n'):
-                print >>to_file,  '      ' + l
+                to_file.write('      %s\n' % (l,))
 
         # TODO: Why not show the modified files in a shorter form as
         # well? rewrap them single lines of appropriate length
         if revision.delta is not None:
             revision.delta.show(to_file, self.show_ids)
-        print >>to_file, ''
+        to_file.write('\n')
 
 
 class LineLogFormatter(LogFormatter):
 
+    supports_single_merge_revision = True
+
     def __init__(self, *args, **kwargs):
-        from bzrlib.osutils import terminal_width
         super(LineLogFormatter, self).__init__(*args, **kwargs)
         self._max_chars = terminal_width() - 1
 
@@ -710,7 +745,6 @@ class LineLogFormatter(LogFormatter):
         return str[:max_len-3]+'...'
 
     def date_string(self, rev):
-        from bzrlib.osutils import format_date
         return format_date(rev.timestamp, rev.timezone or 0, 
                            self.show_timezone, date_fmt="%Y-%m-%d",
                            show_offset=False)
@@ -723,12 +757,13 @@ class LineLogFormatter(LogFormatter):
 
     @deprecated_method(zero_seventeen)
     def show(self, revno, rev, delta):
-        from bzrlib.osutils import terminal_width
-        print >> self.to_file, self.log_string(revno, rev, terminal_width()-1)
+        self.to_file.write(self.log_string(revno, rev, terminal_width()-1))
+        self.to_file.write('\n')
 
     def log_revision(self, revision):
-        print >>self.to_file, self.log_string(revision.revno, revision.rev,
-                                              self._max_chars)
+        self.to_file.write(self.log_string(revision.revno, revision.rev,
+                                              self._max_chars))
+        self.to_file.write('\n')
 
     def log_string(self, revno, rev, max_chars):
         """Format log info into one string. Truncate tail of string
@@ -742,7 +777,7 @@ class LineLogFormatter(LogFormatter):
         if revno:
             # show revno only when is not None
             out.append("%s:" % revno)
-        out.append(self.truncate(self.short_committer(rev), 20))
+        out.append(self.truncate(self.short_author(rev), 20))
         out.append(self.date_string(rev))
         out.append(rev.get_summary())
         return self.truncate(" ".join(out).rstrip('\n'), max_chars)
@@ -789,7 +824,6 @@ def log_formatter(name, *args, **kwargs):
     name -- Name of the formatter to construct; currently 'long', 'short' and
         'line' are supported.
     """
-    from bzrlib.errors import BzrCommandError
     try:
         return log_formatter_registry.make_formatter(name, *args, **kwargs)
     except KeyError:
@@ -802,7 +836,8 @@ def show_one_log(revno, rev, delta, verbose, to_file, show_timezone):
     lf.show(revno, rev, delta)
 
 
-def show_changed_revisions(branch, old_rh, new_rh, to_file=None, log_format='long'):
+def show_changed_revisions(branch, old_rh, new_rh, to_file=None,
+                           log_format='long'):
     """Show the change in revision history comparing the old revision history to the new one.
 
     :param branch: The branch where the revisions exist
@@ -811,10 +846,8 @@ def show_changed_revisions(branch, old_rh, new_rh, to_file=None, log_format='lon
     :param to_file: A file to write the results to. If None, stdout will be used
     """
     if to_file is None:
-        import sys
-        import codecs
-        import bzrlib
-        to_file = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
+        to_file = codecs.getwriter(get_terminal_encoding())(sys.stdout,
+            errors='replace')
     lf = log_formatter(log_format,
                        show_ids=False,
                        to_file=to_file,
@@ -851,7 +884,7 @@ def show_changed_revisions(branch, old_rh, new_rh, to_file=None, log_format='lon
         show_log(branch,
                  lf,
                  None,
-                 verbose=True,
+                 verbose=False,
                  direction='forward',
                  start_revision=base_idx+1,
                  end_revision=len(new_rh),
