@@ -16,10 +16,23 @@
 
 """Display what revisions are missing in 'other' from 'this' and vice versa."""
 
-import bzrlib.ui as ui
+from bzrlib import ui
+from bzrlib.log import (
+    LogRevision,
+    )
+from bzrlib.symbol_versioning import (
+    deprecated_function,
+    zero_seventeen,
+    )
 
 
+@deprecated_function(zero_seventeen)
 def iter_log_data(revisions, revision_source, verbose):
+    for revision in iter_log_revisions(revisions, revision_source, verbose):
+        yield revision.revno, revision.rev, revision.delta
+
+
+def iter_log_revisions(revisions, revision_source, verbose):
     last_tree = revision_source.revision_tree(None)
     last_rev_id = None
     for revno, rev_id in revisions:
@@ -37,7 +50,7 @@ def iter_log_data(revisions, revision_source, verbose):
             delta = revision_tree.changes_from(parent_tree)
         else:
             delta = None
-        yield revno, rev, delta
+        yield LogRevision(rev, revno, delta=delta)
 
 
 def find_unmerged(local_branch, remote_branch):
@@ -46,6 +59,10 @@ def find_unmerged(local_branch, remote_branch):
     try:
         remote_branch.lock_read()
         try:
+            # check for special case: both branches are equivalent
+            if (local_branch.last_revision_info() ==
+                remote_branch.last_revision_info()):
+                return [], []
             local_rev_history, local_rev_history_map = \
                 _get_history(local_branch, progress, "local", 0)
             remote_rev_history, remote_rev_history_map = \
@@ -106,7 +123,8 @@ def _get_history(branch, progress, label, step):
 def _get_ancestry(repository, progress, label, step, rev_history):
     progress.update('%s ancestry' % label, step, 5)
     if len(rev_history) > 0:
-        ancestry = set(repository.get_ancestry(rev_history[-1]))
+        ancestry = set(repository.get_ancestry(rev_history[-1],
+                       topo_sorted=False))
     else:
         ancestry = set()
     return ancestry
