@@ -33,12 +33,12 @@ class TestLogFormats(TestCaseInTempDir):
 
         self.bzr('init')
         open('a', 'wb').write('foo\n')
-        self.bzr('add', 'a')
+        self.bzr('add a')
 
-        self.bzr('commit', '-m', '1')
+        self.bzr('commit -m 1')
         open('a', 'wb').write('baz\n')
 
-        self.bzr('commit', '-m', '2')
+        self.bzr('commit -m 2')
 
         # only the lines formatter is this short
         self.assertEquals(3, len(self.bzr('log').split('\n')))
@@ -46,15 +46,15 @@ class TestLogFormats(TestCaseInTempDir):
     def test_log_format_arg(self):
         self.bzr('init')
         open('a', 'wb').write('foo\n')
-        self.bzr('add', 'a')
+        self.bzr('add a')
 
-        self.bzr('commit', '-m', '1')
+        self.bzr('commit -m 1')
         open('a', 'wb').write('baz\n')
 
-        self.bzr('commit', '-m', '2')
+        self.bzr('commit -m 2')
 
         # only the lines formatter is this short
-        self.assertEquals(7, len(self.bzr('log', '--log-format', 'short').split('\n')))
+        self.assertEquals(7, len(self.bzr('log --log-format short').split('\n')))
 
     def test_missing_default_format(self):
         self.setup_config()
@@ -64,23 +64,23 @@ class TestLogFormats(TestCaseInTempDir):
         self.bzr('init')
 
         open('a', 'wb').write('foo\n')
-        self.bzr('add', 'a')
-        self.bzr('commit', '-m', '1')
+        self.bzr('add a')
+        self.bzr('commit -m 1')
 
         os.chdir('..')
-        self.bzr('branch', 'a', 'b')
+        self.bzr('branch a b')
         os.chdir('a')
 
         open('a', 'wb').write('bar\n')
-        self.bzr('commit', '-m', '2')
+        self.bzr('commit -m 2')
 
         open('a', 'wb').write('baz\n')
-        self.bzr('commit', '-m', '3')
+        self.bzr('commit -m 3')
 
         os.chdir('../b')
-        
+
         self.assertEquals(5, len(self.bzr('missing', retcode=1).split('\n')))
-        
+
         os.chdir('..')
 
     def test_missing_format_arg(self):
@@ -91,23 +91,24 @@ class TestLogFormats(TestCaseInTempDir):
         self.bzr('init')
 
         open('a', 'wb').write('foo\n')
-        self.bzr('add', 'a')
-        self.bzr('commit', '-m', '1')
+        self.bzr('add a')
+        self.bzr('commit -m 1')
 
         os.chdir('..')
-        self.bzr('branch', 'a', 'b')
+        self.bzr('branch a b')
         os.chdir('a')
 
         open('a', 'wb').write('bar\n')
-        self.bzr('commit', '-m', '2')
+        self.bzr('commit -m 2')
 
         open('a', 'wb').write('baz\n')
-        self.bzr('commit', '-m', '3')
+        self.bzr('commit -m 3')
 
         os.chdir('../b')
-        
-        self.assertEquals(9, len(self.bzr('missing', '--log-format', 'short', retcode=1).split('\n')))
-        
+
+        self.assertEquals(9, len(self.bzr('missing --log-format short',
+                                          retcode=1).split('\n')))
+
         os.chdir('..')
 
 
