@@ -49,7 +49,7 @@ from bzrlib.osutils import (
     pathjoin,
     terminal_width,
     )
-from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.tests.http_utils import TestCaseWithWebserver
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.workingtree import WorkingTree
@@ -386,8 +386,8 @@ class OldTests(ExternalBase):
         chdir('branch1')
         self.run_bzr('init')
 
-        self.assertEquals(self.run_bzr('root')[0].rstrip(),
-                          pathjoin(self.test_dir, 'branch1'))
+        self.assertIsSameRealPath(self.run_bzr('root')[0].rstrip(),
+                                  pathjoin(self.test_dir, 'branch1'))
 
         progress("status of new file")
 
