@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ from bzrlib import (
     transactions,
     remote,
     repository,
+    tests,
     )
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 from bzrlib.delta import TreeDelta
@@ -41,31 +42,19 @@ from bzrlib.osutils import getcwd
 import bzrlib.revision
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.tests.branch_implementations import TestCaseWithBranch
-from bzrlib.tests.HttpServer import HttpServer
+from bzrlib.tests.http_server import HttpServer
 from bzrlib.trace import mutter
 from bzrlib.transport import get_transport
 from bzrlib.transport.memory import MemoryServer
 from bzrlib.upgrade import upgrade
 from bzrlib.workingtree import WorkingTree
+from bzrlib.symbol_versioning import (
+    zero_ninetyone,
+    )
 
 
 class TestBranch(TestCaseWithBranch):
 
-    def test_append_revisions(self):
-        """Test appending more than one revision"""
-        wt = self.make_branch_and_tree('tree')
-        wt.commit('f', rev_id='rev1')
-        wt.commit('f', rev_id='rev2')
-        wt.commit('f', rev_id='rev3')
-
-        br = self.get_branch()
-        br.fetch(wt.branch)
-        br.append_revision("rev1")
-        self.assertEquals(br.revision_history(), ["rev1",])
-        br.append_revision("rev2", "rev3")
-        self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
-        self.assertRaises(errors.ReservedId, br.append_revision, 'current:')
-
     def test_create_tree_with_merge(self):
         tree = self.create_tree_with_merge()
         ancestry_graph = tree.branch.repository.get_revision_graph('rev-3')
@@ -109,6 +98,8 @@ class TestBranch(TestCaseWithBranch):
 
         rev = b2.repository.get_revision('revision-1')
         tree = b2.repository.revision_tree('revision-1')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
         self.assertEqual(tree.get_file_text('foo-id'), 'hello')
 
     def test_get_revision_delta(self):
@@ -179,7 +170,7 @@ class TestBranch(TestCaseWithBranch):
 
     def test_clone_branch_nickname(self):
         # test the nick name is preserved always
-        raise TestSkipped('XXX branch cloning is not yet tested..')
+        raise TestSkipped('XXX branch cloning is not yet tested.')
 
     def test_clone_branch_parent(self):
         # test the parent is preserved always
@@ -223,6 +214,8 @@ class TestBranch(TestCaseWithBranch):
         wt = self.make_branch_and_tree('.')
         wt.set_parent_ids(['non:existent@rev--ision--0--2'],
             allow_leftmost_as_ghost=True)
+        self.assertEqual(['non:existent@rev--ision--0--2'],
+            wt.get_parent_ids())
         rev_id = wt.commit('commit against a ghost first parent.')
         rev = wt.branch.repository.get_revision(rev_id)
         self.assertEqual(rev.parent_ids, ['non:existent@rev--ision--0--2'])
@@ -260,7 +253,11 @@ class TestBranch(TestCaseWithBranch):
         wt.commit("base", allow_pointless=True, rev_id='A')
         from bzrlib.testament import Testament
         strategy = gpg.LoopbackGPGStrategy(None)
+        branch.repository.lock_write()
+        branch.repository.start_write_group()
         branch.repository.sign_revision('A', strategy)
+        branch.repository.commit_write_group()
+        branch.repository.unlock()
         self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n' +
                          Testament.from_revision(branch.repository,
                          'A').as_short_text() +
@@ -270,8 +267,19 @@ class TestBranch(TestCaseWithBranch):
     def test_store_signature(self):
         wt = self.make_branch_and_tree('.')
         branch = wt.branch
-        branch.repository.store_revision_signature(
-            gpg.LoopbackGPGStrategy(None), 'FOO', 'A')
+        branch.lock_write()
+        try:
+            branch.repository.start_write_group()
+            try:
+                branch.repository.store_revision_signature(
+                    gpg.LoopbackGPGStrategy(None), 'FOO', 'A')
+            except:
+                branch.repository.abort_write_group()
+                raise
+            else:
+                branch.repository.commit_write_group()
+        finally:
+            branch.unlock()
         self.assertRaises(errors.NoSuchRevision,
                           branch.repository.has_signature_for_revision_id,
                           'A')
@@ -284,7 +292,11 @@ class TestBranch(TestCaseWithBranch):
         wt = self.make_branch_and_tree('source')
         wt.commit('A', allow_pointless=True, rev_id='A')
         repo = wt.branch.repository
+        repo.lock_write()
+        repo.start_write_group()
         repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
+        repo.commit_write_group()
+        repo.unlock()
         #FIXME: clone should work to urls,
         # wt.clone should work to disks.
         self.build_tree(['target/'])
@@ -393,7 +405,7 @@ class TestBranch(TestCaseWithBranch):
         tree_a = self.make_branch_and_tree('a')
         branch_a = tree_a.branch
         checkout_b = branch_a.create_checkout('b')
-        self.assertEqual(None, checkout_b.last_revision())
+        self.assertEqual('null:', checkout_b.last_revision())
         checkout_b.commit('rev1', rev_id='rev1')
         self.assertEqual('rev1', branch_a.last_revision())
         self.assertNotEqual(checkout_b.branch.base, branch_a.base)
@@ -543,9 +555,9 @@ class TestBranchPushLocations(TestCaseWithBranch):
                                    ensure_config_dir_exists)
         ensure_config_dir_exists()
         fn = locations_config_filename()
-        print >> open(fn, 'wt'), ("[%s]\n"
-                                  "push_location=foo" %
-                                  self.get_branch().base[:-1])
+        open(fn, 'wt').write(("[%s]\n"
+                                  "push_location=foo\n" %
+                                  self.get_branch().base[:-1]))
         self.assertEqual("foo", self.get_branch().get_push_location())
 
     def test_set_push_location(self):
@@ -568,6 +580,24 @@ class TestFormat(TestCaseWithBranch):
         self.assertEqual(None,
             made_branch._format.get_reference(made_branch.bzrdir))
 
+    def test_set_reference(self):
+        """set_reference on all regular branches should be callable."""
+        if not self.branch_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        this_branch = self.make_branch('this')
+        other_branch = self.make_branch('other')
+        try:
+            this_branch._format.set_reference(this_branch.bzrdir, other_branch)
+        except NotImplementedError:
+            # that's ok
+            pass
+        else:
+            ref = this_branch._format.get_reference(this_branch.bzrdir)
+            self.assertEqual(ref, other_branch.base)
+
     def test_format_initialize_find_open(self):
         # loopback test to check the current format initializes to itself.
         if not self.branch_format.is_supported():
@@ -611,7 +641,7 @@ class TestBound(TestCaseWithBranch):
         try:
             branch.bind(branch2)
         except errors.UpgradeRequired:
-            raise TestSkipped('Format does not support binding')
+            raise tests.TestNotApplicable('Format does not support binding')
         self.assertTrue(branch.unbind())
         self.assertFalse(branch.unbind())
         self.assertIs(None, branch.get_bound_location())
@@ -621,13 +651,25 @@ class TestBound(TestCaseWithBranch):
         try:
             self.assertIs(None, branch.get_old_bound_location())
         except errors.UpgradeRequired:
-            raise TestSkipped('Format does not store old bound locations')
+            raise tests.TestNotApplicable(
+                    'Format does not store old bound locations')
         branch2 = self.make_branch('branch2')
         branch.bind(branch2)
         self.assertIs(None, branch.get_old_bound_location())
         branch.unbind()
         self.assertContainsRe(branch.get_old_bound_location(), '\/branch2\/$')
 
+    def test_bind_diverged(self):
+        tree_a = self.make_branch_and_tree('tree_a')
+        tree_a.commit('rev1a')
+        tree_b = tree_a.bzrdir.sprout('tree_b').open_workingtree()
+        tree_a.commit('rev2a')
+        tree_b.commit('rev2b')
+        try:
+            tree_b.branch.bind(tree_a.branch)
+        except errors.UpgradeRequired:
+            raise tests.TestNotApplicable('Format does not support binding')
+
 
 class TestStrict(TestCaseWithBranch):
 
