@@ -26,23 +26,12 @@ from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
 class TestCreateCheckout(TestCaseWithBranch):
 
     def test_checkout_format(self):
-        """Make sure the new checkout uses the same branch format."""
+        """Make sure the new checkout uses the desired branch format."""
         a_branch = self.make_branch('branch')
-        if isinstance(a_branch, RemoteBranch):
-            # RemoteBranch formats are not the same as local ones, and dont
-            # duplicate the format string (because there is no format string as
-            # such - it might be e.g. totally virtual on the server end).
-            # This test can only assess the checkout format correctness *in
-            # general* when there is a real object locally present for both the
-            # source and target.
-            return
         tree = a_branch.create_checkout('checkout')
-        if self.branch_format in branch._legacy_formats:
-            # Legacy formats create checkouts with the default format.
-            # Only newer formats create identical checkouts.
-            expected_format = branch.BranchFormat.get_default_format()
-        else:
-            expected_format = a_branch._format
+        # All branches can define the format they want checkouts made in.
+        # This checks it is honoured.
+        expected_format = a_branch._get_checkout_format().get_branch_format()
         self.assertEqual(expected_format.__class__,
                          tree.branch._format.__class__)
 
@@ -71,5 +60,5 @@ class TestCreateCheckout(TestCaseWithBranch):
     def test_create_checkout_exists(self):
         """We shouldn't fail if the directory already exists."""
         tree1 = self.make_branch_and_tree('base')
-        self.build_tree('checkout')
+        self.build_tree(['checkout/'])
         tree2 = tree1.branch.create_checkout('checkout', lightweight=True)
