@@ -16,7 +16,6 @@
 
 
 from bzrlib.builtins import cmd_missing
-from bzrlib.tests import StringIOWrapper
 from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
 
 
@@ -29,11 +28,11 @@ class TestMissing(TestCaseWithConnectionHookedTransport):
         wt2.pull(wt1.branch)
         wt2.commit('empty commit too')
 
-        self.install_hooks()
+        self.start_logging_connections()
 
         cmd = cmd_missing()
         # We don't care about the ouput but 'outf' should be defined
-        cmd.outf = StringIOWrapper()
+        cmd.outf = self.make_utf8_encoded_stringio()
         cmd.run(self.get_url('branch2'))
         self.assertEquals(1, len(self.connections))
 
