@@ -20,6 +20,7 @@
 # affects the global state of the process.  See bzrlib/plugins.py for more
 # comments.
 
+import logging
 import os
 from StringIO import StringIO
 import sys
@@ -30,8 +31,9 @@ import bzrlib.plugin
 import bzrlib.plugins
 import bzrlib.commands
 import bzrlib.help
+from bzrlib.symbol_versioning import one_three
 from bzrlib.tests import TestCase, TestCaseInTempDir
-from bzrlib.osutils import pathjoin, abspath
+from bzrlib.osutils import pathjoin, abspath, normpath
 
 
 PLUGIN_TEXT = """\
@@ -69,14 +71,29 @@ class TestLoadingPlugins(TestCaseInTempDir):
         # tempattribute list.
         template = ("from bzrlib.tests.test_plugins import TestLoadingPlugins\n"
                     "TestLoadingPlugins.activeattributes[%r].append('%s')\n")
-        print >> file(os.path.join('first', 'plugin.py'), 'w'), template % (tempattribute, 'first')
-        print >> file(os.path.join('second', 'plugin.py'), 'w'), template % (tempattribute, 'second')
+
+        outfile = open(os.path.join('first', 'plugin.py'), 'w')
+        try:
+            outfile.write(template % (tempattribute, 'first'))
+            outfile.write('\n')
+        finally:
+            outfile.close()
+
+        outfile = open(os.path.join('second', 'plugin.py'), 'w')
+        try:
+            outfile.write(template % (tempattribute, 'second'))
+            outfile.write('\n')
+        finally:
+            outfile.close()
+
         try:
             bzrlib.plugin.load_from_path(['first', 'second'])
             self.assertEqual(['first'], self.activeattributes[tempattribute])
         finally:
             # remove the plugin 'plugin'
             del self.activeattributes[tempattribute]
+            if 'bzrlib.plugins.plugin' in sys.modules:
+                del sys.modules['bzrlib.plugins.plugin']
             if getattr(bzrlib.plugins, 'plugin', None):
                 del bzrlib.plugins.plugin
         self.failIf(getattr(bzrlib.plugins, 'plugin', None))
@@ -102,8 +119,21 @@ class TestLoadingPlugins(TestCaseInTempDir):
         # tempattribute list.
         template = ("from bzrlib.tests.test_plugins import TestLoadingPlugins\n"
                     "TestLoadingPlugins.activeattributes[%r].append('%s')\n")
-        print >> file(os.path.join('first', 'pluginone.py'), 'w'), template % (tempattribute, 'first')
-        print >> file(os.path.join('second', 'plugintwo.py'), 'w'), template % (tempattribute, 'second')
+
+        outfile = open(os.path.join('first', 'pluginone.py'), 'w')
+        try:
+            outfile.write(template % (tempattribute, 'first'))
+            outfile.write('\n')
+        finally:
+            outfile.close()
+
+        outfile = open(os.path.join('second', 'plugintwo.py'), 'w')
+        try:
+            outfile.write(template % (tempattribute, 'second'))
+            outfile.write('\n')
+        finally:
+            outfile.close()
+
         oldpath = bzrlib.plugins.__path__
         try:
             bzrlib.plugins.__path__ = ['first', 'second']
@@ -115,33 +145,175 @@ class TestLoadingPlugins(TestCaseInTempDir):
         finally:
             # remove the plugin 'plugin'
             del self.activeattributes[tempattribute]
-            if getattr(bzrlib.plugins, 'plugin', None):
-                del bzrlib.plugins.plugin
-        self.failIf(getattr(bzrlib.plugins, 'plugin', None))
+            if getattr(bzrlib.plugins, 'pluginone', None):
+                del bzrlib.plugins.pluginone
+            if getattr(bzrlib.plugins, 'plugintwo', None):
+                del bzrlib.plugins.plugintwo
+        self.failIf(getattr(bzrlib.plugins, 'pluginone', None))
+        self.failIf(getattr(bzrlib.plugins, 'plugintwo', None))
+
+    def test_plugins_can_load_from_directory_with_trailing_slash(self):
+        # This test tests that a plugin can load from a directory when the
+        # directory in the path has a trailing slash.
+        # check the plugin is not loaded already
+        self.failIf(getattr(bzrlib.plugins, 'ts_plugin', None))
+        tempattribute = "trailing-slash"
+        self.failIf(tempattribute in self.activeattributes)
+        # set a place for the plugin to record its loading, and at the same
+        # time validate that the location the plugin should record to is
+        # valid and correct.
+        bzrlib.tests.test_plugins.TestLoadingPlugins.activeattributes \
+            [tempattribute] = []
+        self.failUnless(tempattribute in self.activeattributes)
+        # create a directory for the plugin
+        os.mkdir('plugin_test')
+        # write a plugin that will record when its loaded in the 
+        # tempattribute list.
+        template = ("from bzrlib.tests.test_plugins import TestLoadingPlugins\n"
+                    "TestLoadingPlugins.activeattributes[%r].append('%s')\n")
+
+        outfile = open(os.path.join('plugin_test', 'ts_plugin.py'), 'w')
+        try:
+            outfile.write(template % (tempattribute, 'plugin'))
+            outfile.write('\n')
+        finally:
+            outfile.close()
+
+        try:
+            bzrlib.plugin.load_from_path(['plugin_test'+os.sep])
+            self.assertEqual(['plugin'], self.activeattributes[tempattribute])
+        finally:
+            # remove the plugin 'plugin'
+            del self.activeattributes[tempattribute]
+            if getattr(bzrlib.plugins, 'ts_plugin', None):
+                del bzrlib.plugins.ts_plugin
+        self.failIf(getattr(bzrlib.plugins, 'ts_plugin', None))
+
+    def test_plugin_with_bad_name_does_not_load(self):
+        # Create badly-named plugin
+        file('bad plugin-name..py', 'w').close()
+
+        # Capture output
+        stream = StringIO()
+        handler = logging.StreamHandler(stream)
+        log = logging.getLogger('bzr')
+        log.addHandler(handler)
+
+        bzrlib.plugin.load_from_dir('.')
+
+        # Stop capturing output
+        handler.flush()
+        handler.close()
+        log.removeHandler(handler)
+
+        self.assertContainsRe(stream.getvalue(),
+            r"Unable to load 'bad plugin-name\.' in '\.' as a plugin because"
+            " file path isn't a valid module name; try renaming it to"
+            " 'bad_plugin_name_'\.")
+
+        stream.close()
 
 
-class TestAllPlugins(TestCaseInTempDir):
+class TestPlugins(TestCaseInTempDir):
 
-    def test_plugin_appears_in_all_plugins(self):
-        # This test tests a new plugin appears in bzrlib.plugin.all_plugins().
+    def setup_plugin(self, source=""):
+        # This test tests a new plugin appears in bzrlib.plugin.plugins().
         # check the plugin is not loaded already
         self.failIf(getattr(bzrlib.plugins, 'plugin', None))
         # write a plugin that _cannot_ fail to load.
-        print >> file('plugin.py', 'w'), ""
-        try:
-            bzrlib.plugin.load_from_path(['.'])
-            self.failUnless('plugin' in bzrlib.plugin.all_plugins())
-            self.failUnless(getattr(bzrlib.plugins, 'plugin', None))
-            self.assertEqual(bzrlib.plugin.all_plugins()['plugin'],
-                             bzrlib.plugins.plugin)
-        finally:
-            # remove the plugin 'plugin'
-            if 'bzrlib.plugins.plugin' in sys.modules:
-                del sys.modules['bzrlib.plugins.plugin']
-            if getattr(bzrlib.plugins, 'plugin', None):
-                del bzrlib.plugins.plugin
+        file('plugin.py', 'w').write(source + '\n')
+        self.addCleanup(self.teardown_plugin)
+        bzrlib.plugin.load_from_path(['.'])
+    
+    def teardown_plugin(self):
+        # remove the plugin 'plugin'
+        if 'bzrlib.plugins.plugin' in sys.modules:
+            del sys.modules['bzrlib.plugins.plugin']
+        if getattr(bzrlib.plugins, 'plugin', None):
+            del bzrlib.plugins.plugin
         self.failIf(getattr(bzrlib.plugins, 'plugin', None))
 
+    def test_plugin_appears_in_plugins(self):
+        self.setup_plugin()
+        self.failUnless('plugin' in bzrlib.plugin.plugins())
+        self.failUnless(getattr(bzrlib.plugins, 'plugin', None))
+        plugins = bzrlib.plugin.plugins()
+        plugin = plugins['plugin']
+        self.assertIsInstance(plugin, bzrlib.plugin.PlugIn)
+        self.assertEqual(bzrlib.plugins.plugin, plugin.module)
+
+    def test_trivial_plugin_get_path(self):
+        self.setup_plugin()
+        plugins = bzrlib.plugin.plugins()
+        plugin = plugins['plugin']
+        plugin_path = self.test_dir + '/plugin.py'
+        self.assertIsSameRealPath(plugin_path, normpath(plugin.path()))
+
+    def test_plugin_get_path_py_not_pyc(self):
+        self.setup_plugin()         # after first import there will be plugin.pyc
+        self.teardown_plugin()
+        bzrlib.plugin.load_from_path(['.']) # import plugin.pyc
+        plugins = bzrlib.plugin.plugins()
+        plugin = plugins['plugin']
+        plugin_path = self.test_dir + '/plugin.py'
+        self.assertIsSameRealPath(plugin_path, normpath(plugin.path()))
+
+    def test_plugin_get_path_pyc_only(self):
+        self.setup_plugin()         # after first import there will be plugin.pyc
+        self.teardown_plugin()
+        os.unlink(self.test_dir + '/plugin.py')
+        bzrlib.plugin.load_from_path(['.']) # import plugin.pyc
+        plugins = bzrlib.plugin.plugins()
+        plugin = plugins['plugin']
+        if __debug__:
+            plugin_path = self.test_dir + '/plugin.pyc'
+        else:
+            plugin_path = self.test_dir + '/plugin.pyo'
+        self.assertIsSameRealPath(plugin_path, normpath(plugin.path()))
+
+    def test_no_test_suite_gives_None_for_test_suite(self):
+        self.setup_plugin()
+        plugin = bzrlib.plugin.plugins()['plugin']
+        self.assertEqual(None, plugin.test_suite())
+
+    def test_test_suite_gives_test_suite_result(self):
+        source = """def test_suite(): return 'foo'"""
+        self.setup_plugin(source)
+        plugin = bzrlib.plugin.plugins()['plugin']
+        self.assertEqual('foo', plugin.test_suite())
+
+    def test_no_version_info(self):
+        self.setup_plugin()
+        plugin = bzrlib.plugin.plugins()['plugin']
+        self.assertEqual(None, plugin.version_info())
+
+    def test_with_version_info(self):
+        self.setup_plugin("version_info = (1, 2, 3, 'dev', 4)")
+        plugin = bzrlib.plugin.plugins()['plugin']
+        self.assertEqual((1, 2, 3, 'dev', 4), plugin.version_info())
+
+    def test_short_version_info_gets_padded(self):
+        # the gtk plugin has version_info = (1,2,3) rather than the 5-tuple.
+        # so we adapt it
+        self.setup_plugin("version_info = (1, 2, 3)")
+        plugin = bzrlib.plugin.plugins()['plugin']
+        self.assertEqual((1, 2, 3, 'final', 0), plugin.version_info())
+
+    def test_no_version_info___version__(self):
+        self.setup_plugin()
+        plugin = bzrlib.plugin.plugins()['plugin']
+        self.assertEqual("unknown", plugin.__version__)
+
+    def test___version__with_version_info(self):
+        self.setup_plugin("version_info = (1, 2, 3, 'dev', 4)")
+        plugin = bzrlib.plugin.plugins()['plugin']
+        self.assertEqual("1.2.3dev4", plugin.__version__)
+
+    def test_final__version__with_version_info(self):
+        self.setup_plugin("version_info = (1, 2, 3, 'final', 4)")
+        plugin = bzrlib.plugin.plugins()['plugin']
+        self.assertEqual("1.2.3", plugin.__version__)
+
 
 class TestPluginHelp(TestCaseInTempDir):
 
@@ -212,7 +384,8 @@ class TestPluginFromZip(TestCaseInTempDir):
         try:
             # this is normally done by load_plugins -> set_plugins_path
             bzrlib.plugins.__path__ = [zip_name]
-            bzrlib.plugin.load_from_zip(zip_name)
+            self.applyDeprecated(one_three,
+                bzrlib.plugin.load_from_zip, zip_name)
             self.assertTrue(plugin_name in dir(bzrlib.plugins),
                             'Plugin is not loaded')
         finally:
@@ -243,6 +416,25 @@ class TestSetPluginsPath(TestCase):
         finally:
             bzrlib.plugins.__path__ = old_path
 
+    def test_set_plugins_path_with_trailing_slashes(self):
+        """set_plugins_path should set the module __path__ based on
+        BZR_PLUGIN_PATH."""
+        old_path = bzrlib.plugins.__path__
+        old_env = os.environ.get('BZR_PLUGIN_PATH')
+        try:
+            bzrlib.plugins.__path__ = []
+            os.environ['BZR_PLUGIN_PATH'] = "first\\//\\" + os.pathsep + \
+                "second/\\/\\/"
+            bzrlib.plugin.set_plugins_path()
+            expected_path = ['first', 'second',
+                os.path.dirname(bzrlib.plugins.__file__)]
+            self.assertEqual(expected_path, bzrlib.plugins.__path__)
+        finally:
+            bzrlib.plugins.__path__ = old_path
+            if old_env != None:
+                os.environ['BZR_PLUGIN_PATH'] = old_env
+            else:
+                del os.environ['BZR_PLUGIN_PATH']
 
 class TestHelpIndex(tests.TestCase):
     """Tests for the PluginsHelpIndex class."""
