@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 by Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@ but this depends on the transport.
 
 import cStringIO
 import os
+import re
 import sys
 from tempfile import TemporaryFile
 
@@ -353,3 +354,39 @@ s: both\\\"
         self.assertEqual(u'foo: %s\n' % uni_data, child_text)
         new_child = rio.read_stanza_unicode(child_text.splitlines(True))
         self.assertEqual(uni_data, new_child.get('foo'))
+
+    def mail_munge(self, lines, dos_nl=True):
+        new_lines = []
+        for line in lines:
+            line = re.sub(' *\n', '\n', line)
+            if dos_nl:
+                line = re.sub('([^\r])\n', '\\1\r\n', line)
+            new_lines.append(line)
+        return new_lines
+
+    def test_patch_rio(self):
+        stanza = Stanza(data='#\n\r\\r ', space=' ' * 255, hash='#' * 255)
+        lines = rio.to_patch_lines(stanza)
+        for line in lines:
+            self.assertContainsRe(line, '^# ')
+            self.assertTrue(72 >= len(line))
+        for line in rio.to_patch_lines(stanza, max_width=12):
+            self.assertTrue(12 >= len(line))
+        new_stanza = rio.read_patch_stanza(self.mail_munge(lines,
+                                                           dos_nl=False))
+        lines = self.mail_munge(lines)
+        new_stanza = rio.read_patch_stanza(lines)
+        self.assertEqual('#\n\r\\r ', new_stanza.get('data'))
+        self.assertEqual(' '* 255, new_stanza.get('space'))
+        self.assertEqual('#'* 255, new_stanza.get('hash'))
+
+    def test_patch_rio_linebreaks(self):
+        stanza = Stanza(breaktest='linebreak -/'*30)
+        self.assertContainsRe(rio.to_patch_lines(stanza, 71)[0],
+                              'linebreak\\\\\n')
+        stanza = Stanza(breaktest='linebreak-/'*30)
+        self.assertContainsRe(rio.to_patch_lines(stanza, 70)[0],
+                              'linebreak-\\\\\n')
+        stanza = Stanza(breaktest='linebreak/'*30)
+        self.assertContainsRe(rio.to_patch_lines(stanza, 70)[0],
+                              'linebreak\\\\\n')
