@@ -14,17 +14,59 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Tests for the smart wire/domain protocol."""
+"""Tests for the smart wire/domain protocol.
 
+This module contains tests for the domain-level smart requests and responses,
+such as the 'Branch.lock_write' request. Many of these use specific disk
+formats to exercise calls that only make sense for formats with specific
+properties.
+
+Tests for low-level protocol encoding are found in test_smart_transport.
+"""
+
+import bz2
 from StringIO import StringIO
 import tempfile
 import tarfile
 
-from bzrlib import bzrdir, errors, smart, tests
-from bzrlib.smart.request import SmartServerResponse
+from bzrlib import bzrdir, errors, pack, smart, tests
+from bzrlib.smart.request import (
+    FailedSmartServerResponse,
+    SmartServerResponse,
+    SuccessfulSmartServerResponse,
+    )
 import bzrlib.smart.bzrdir
 import bzrlib.smart.branch
 import bzrlib.smart.repository
+from bzrlib.tests import (
+    iter_suite_tests,
+    split_suite_by_re,
+    TestScenarioApplier,
+    )
+from bzrlib.util import bencode
+
+
+def load_tests(standard_tests, module, loader):
+    """Multiply tests version and protocol consistency."""
+    # FindRepository tests.
+    bzrdir_mod = bzrlib.smart.bzrdir
+    applier = TestScenarioApplier()
+    applier.scenarios = [
+        ("find_repository", {
+            "_request_class":bzrdir_mod.SmartServerRequestFindRepositoryV1}),
+        ("find_repositoryV2", {
+            "_request_class":bzrdir_mod.SmartServerRequestFindRepositoryV2}),
+        ]
+    to_adapt, result = split_suite_by_re(standard_tests,
+        "TestSmartServerRequestFindRepository")
+    v2_only, v1_and_2 = split_suite_by_re(to_adapt,
+        "_v2")
+    for test in iter_suite_tests(v1_and_2):
+        result.addTests(applier.adapt(test))
+    del applier.scenarios[0]
+    for test in iter_suite_tests(v2_only):
+        result.addTests(applier.adapt(test))
+    return result
 
 
 class TestCaseWithSmartMedium(tests.TestCaseWithTransport):
@@ -63,7 +105,7 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
     def test_no_repository(self):
         """When there is no repository to be found, ('norepository', ) is returned."""
         backing = self.get_transport()
-        request = smart.bzrdir.SmartServerRequestFindRepository(backing)
+        request = self._request_class(backing)
         self.make_bzrdir('.')
         self.assertEqual(SmartServerResponse(('norepository', )),
             request.execute(backing.local_abspath('')))
@@ -73,7 +115,7 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
         # path the repository is being searched on is the same as that that 
         # the repository is at.
         backing = self.get_transport()
-        request = smart.bzrdir.SmartServerRequestFindRepository(backing)
+        request = self._request_class(backing)
         result = self._make_repository_and_result()
         self.assertEqual(result, request.execute(backing.local_abspath('')))
         self.make_bzrdir('subdir')
@@ -94,12 +136,19 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
             subtrees = 'yes'
         else:
             subtrees = 'no'
-        return SmartServerResponse(('ok', '', rich_root, subtrees))
+        if (smart.bzrdir.SmartServerRequestFindRepositoryV2 ==
+            self._request_class):
+            # All tests so far are on formats, and for non-external
+            # repositories.
+            return SuccessfulSmartServerResponse(
+                ('ok', '', rich_root, subtrees, 'no'))
+        else:
+            return SuccessfulSmartServerResponse(('ok', '', rich_root, subtrees))
 
     def test_shared_repository(self):
         """When there is a shared repository, we get 'ok', 'relpath-to-repo'."""
         backing = self.get_transport()
-        request = smart.bzrdir.SmartServerRequestFindRepository(backing)
+        request = self._request_class(backing)
         result = self._make_repository_and_result(shared=True)
         self.assertEqual(result, request.execute(backing.local_abspath('')))
         self.make_bzrdir('subdir')
@@ -114,13 +163,22 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
     def test_rich_root_and_subtree_encoding(self):
         """Test for the format attributes for rich root and subtree support."""
         backing = self.get_transport()
-        request = smart.bzrdir.SmartServerRequestFindRepository(backing)
+        request = self._request_class(backing)
         result = self._make_repository_and_result(format='dirstate-with-subtree')
         # check the test will be valid
         self.assertEqual('yes', result.args[2])
         self.assertEqual('yes', result.args[3])
         self.assertEqual(result, request.execute(backing.local_abspath('')))
 
+    def test_supports_external_lookups_no_v2(self):
+        """Test for the supports_external_lookups attribute."""
+        backing = self.get_transport()
+        request = self._request_class(backing)
+        result = self._make_repository_and_result(format='dirstate-with-subtree')
+        # check the test will be valid
+        self.assertEqual('no', result.args[4])
+        self.assertEqual(result, request.execute(backing.local_abspath('')))
+
 
 class TestSmartServerRequestInitializeBzrDir(tests.TestCaseWithTransport):
 
@@ -371,7 +429,7 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
     def test_lock_write_on_unlocked_branch(self):
         backing = self.get_transport()
         request = smart.branch.SmartServerBranchRequestLockWrite(backing)
-        branch = self.make_branch('.')
+        branch = self.make_branch('.', format='knit')
         repository = branch.repository
         response = request.execute(backing.local_abspath(''))
         branch_nonce = branch.control_files._lock.peek().get('nonce')
@@ -398,7 +456,7 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
     def test_lock_write_with_tokens_on_locked_branch(self):
         backing = self.get_transport()
         request = smart.branch.SmartServerBranchRequestLockWrite(backing)
-        branch = self.make_branch('.')
+        branch = self.make_branch('.', format='knit')
         branch_token = branch.lock_write()
         repo_token = branch.repository.lock_write()
         branch.repository.unlock()
@@ -413,7 +471,7 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
     def test_lock_write_with_mismatched_tokens_on_locked_branch(self):
         backing = self.get_transport()
         request = smart.branch.SmartServerBranchRequestLockWrite(backing)
-        branch = self.make_branch('.')
+        branch = self.make_branch('.', format='knit')
         branch_token = branch.lock_write()
         repo_token = branch.repository.lock_write()
         branch.repository.unlock()
@@ -428,7 +486,7 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
     def test_lock_write_on_locked_repo(self):
         backing = self.get_transport()
         request = smart.branch.SmartServerBranchRequestLockWrite(backing)
-        branch = self.make_branch('.')
+        branch = self.make_branch('.', format='knit')
         branch.repository.lock_write()
         branch.repository.leave_lock_in_place()
         branch.repository.unlock()
@@ -441,8 +499,9 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
         request = smart.branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.')
         response = request.execute('')
-        self.assertEqual(
-            SmartServerResponse(('UnlockableTransport',)), response)
+        error_name, lock_str, why_str = response.args
+        self.assertFalse(response.is_successful())
+        self.assertEqual('LockFailed', error_name)
 
 
 class TestSmartServerBranchRequestUnlock(tests.TestCaseWithTransport):
@@ -454,7 +513,7 @@ class TestSmartServerBranchRequestUnlock(tests.TestCaseWithTransport):
     def test_unlock_on_locked_branch_and_repo(self):
         backing = self.get_transport()
         request = smart.branch.SmartServerBranchRequestUnlock(backing)
-        branch = self.make_branch('.')
+        branch = self.make_branch('.', format='knit')
         # Lock the branch
         branch_token = branch.lock_write()
         repo_token = branch.repository.lock_write()
@@ -477,7 +536,7 @@ class TestSmartServerBranchRequestUnlock(tests.TestCaseWithTransport):
     def test_unlock_on_unlocked_branch_unlocked_repo(self):
         backing = self.get_transport()
         request = smart.branch.SmartServerBranchRequestUnlock(backing)
-        branch = self.make_branch('.')
+        branch = self.make_branch('.', format='knit')
         response = request.execute(
             backing.local_abspath(''), 'branch token', 'repo token')
         self.assertEqual(
@@ -486,7 +545,7 @@ class TestSmartServerBranchRequestUnlock(tests.TestCaseWithTransport):
     def test_unlock_on_unlocked_branch_locked_repo(self):
         backing = self.get_transport()
         request = smart.branch.SmartServerBranchRequestUnlock(backing)
-        branch = self.make_branch('.')
+        branch = self.make_branch('.', format='knit')
         # Lock the repository.
         repo_token = branch.repository.lock_write()
         branch.repository.leave_lock_in_place()
@@ -515,6 +574,22 @@ class TestSmartServerRepositoryRequest(tests.TestCaseWithTransport):
             request.execute, backing.local_abspath('subdir'))
 
 
+class TestSmartServerRepositoryGetParentMap(tests.TestCaseWithTransport):
+
+    def test_trivial_bzipped(self):
+        # This tests that the wire encoding is actually bzipped
+        backing = self.get_transport()
+        request = smart.repository.SmartServerRepositoryGetParentMap(backing)
+        tree = self.make_branch_and_memory_tree('.')
+
+        self.assertEqual(None,
+            request.execute(backing.local_abspath(''), 'missing-id'))
+        # Note that it returns a body (of '' bzipped).
+        self.assertEqual(
+            SuccessfulSmartServerResponse(('ok', ), bz2.compress('')),
+            request.do_body('\n\n0\n'))
+
+
 class TestSmartServerRepositoryGetRevisionGraph(tests.TestCaseWithTransport):
 
     def test_none_argument(self):
@@ -682,7 +757,7 @@ class TestSmartServerRepositoryLockWrite(tests.TestCaseWithTransport):
     def test_lock_write_on_unlocked_repo(self):
         backing = self.get_transport()
         request = smart.repository.SmartServerRepositoryLockWrite(backing)
-        repository = self.make_repository('.')
+        repository = self.make_repository('.', format='knit')
         response = request.execute(backing.local_abspath(''))
         nonce = repository.control_files._lock.peek().get('nonce')
         self.assertEqual(SmartServerResponse(('ok', nonce)), response)
@@ -694,7 +769,7 @@ class TestSmartServerRepositoryLockWrite(tests.TestCaseWithTransport):
     def test_lock_write_on_locked_repo(self):
         backing = self.get_transport()
         request = smart.repository.SmartServerRepositoryLockWrite(backing)
-        repository = self.make_repository('.')
+        repository = self.make_repository('.', format='knit')
         repository.lock_write()
         repository.leave_lock_in_place()
         repository.unlock()
@@ -705,10 +780,10 @@ class TestSmartServerRepositoryLockWrite(tests.TestCaseWithTransport):
     def test_lock_write_on_readonly_transport(self):
         backing = self.get_readonly_transport()
         request = smart.repository.SmartServerRepositoryLockWrite(backing)
-        repository = self.make_repository('.')
+        repository = self.make_repository('.', format='knit')
         response = request.execute('')
-        self.assertEqual(
-            SmartServerResponse(('UnlockableTransport',)), response)
+        self.assertFalse(response.is_successful())
+        self.assertEqual('LockFailed', response.args[0])
 
 
 class TestSmartServerRepositoryUnlock(tests.TestCaseWithTransport):
@@ -720,7 +795,7 @@ class TestSmartServerRepositoryUnlock(tests.TestCaseWithTransport):
     def test_unlock_on_locked_repo(self):
         backing = self.get_transport()
         request = smart.repository.SmartServerRepositoryUnlock(backing)
-        repository = self.make_repository('.')
+        repository = self.make_repository('.', format='knit')
         token = repository.lock_write()
         repository.leave_lock_in_place()
         repository.unlock()
@@ -736,7 +811,7 @@ class TestSmartServerRepositoryUnlock(tests.TestCaseWithTransport):
     def test_unlock_on_unlocked_repo(self):
         backing = self.get_transport()
         request = smart.repository.SmartServerRepositoryUnlock(backing)
-        repository = self.make_repository('.')
+        repository = self.make_repository('.', format='knit')
         response = request.execute(backing.local_abspath(''), 'some token')
         self.assertEqual(
             SmartServerResponse(('TokenMismatch',)), response)
@@ -766,6 +841,89 @@ class TestSmartServerRepositoryTarball(tests.TestCaseWithTransport):
             "extraneous file present in tar file")
 
 
+class TestSmartServerRepositoryStreamKnitData(tests.TestCaseWithTransport):
+
+    def test_fetch_revisions(self):
+        backing = self.get_transport()
+        request = smart.repository.SmartServerRepositoryStreamKnitDataForRevisions(backing)
+        tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
+        rev_id1_utf8 = u'\xc8'.encode('utf-8')
+        rev_id2_utf8 = u'\xc9'.encode('utf-8')
+        r1 = tree.commit('1st commit', rev_id=rev_id1_utf8)
+        r1 = tree.commit('2nd commit', rev_id=rev_id2_utf8)
+        tree.unlock()
+
+        response = request.execute(backing.local_abspath(''), rev_id2_utf8)
+        self.assertEqual(('ok',), response.args)
+        from cStringIO import StringIO
+        unpacker = pack.ContainerReader(StringIO(response.body))
+        names = []
+        for [name], read_bytes in unpacker.iter_records():
+            names.append(name)
+            bytes = read_bytes(None)
+            # The bytes should be a valid bencoded string.
+            bencode.bdecode(bytes)
+            # XXX: assert that the bencoded knit records have the right
+            # contents?
+        
+    def test_no_such_revision_error(self):
+        backing = self.get_transport()
+        request = smart.repository.SmartServerRepositoryStreamKnitDataForRevisions(backing)
+        repo = self.make_repository('.')
+        rev_id1_utf8 = u'\xc8'.encode('utf-8')
+        response = request.execute(backing.local_abspath(''), rev_id1_utf8)
+        self.assertEqual(
+            SmartServerResponse(('NoSuchRevision', rev_id1_utf8)),
+            response)
+
+
+class TestSmartServerRepositoryStreamRevisionsChunked(tests.TestCaseWithTransport):
+
+    def test_fetch_revisions(self):
+        backing = self.get_transport()
+        request = smart.repository.SmartServerRepositoryStreamRevisionsChunked(
+            backing)
+        tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
+        rev_id1_utf8 = u'\xc8'.encode('utf-8')
+        rev_id2_utf8 = u'\xc9'.encode('utf-8')
+        tree.commit('1st commit', rev_id=rev_id1_utf8)
+        tree.commit('2nd commit', rev_id=rev_id2_utf8)
+        tree.unlock()
+
+        response = request.execute(backing.local_abspath(''))
+        self.assertEqual(None, response)
+        response = request.do_body("%s\n%s\n1" % (rev_id2_utf8, rev_id1_utf8))
+        self.assertEqual(('ok',), response.args)
+        from cStringIO import StringIO
+        parser = pack.ContainerPushParser()
+        names = []
+        for stream_bytes in response.body_stream:
+            parser.accept_bytes(stream_bytes)
+            for [name], record_bytes in parser.read_pending_records():
+                names.append(name)
+                # The bytes should be a valid bencoded string.
+                bencode.bdecode(record_bytes)
+                # XXX: assert that the bencoded knit records have the right
+                # contents?
+        
+    def test_no_such_revision_error(self):
+        backing = self.get_transport()
+        request = smart.repository.SmartServerRepositoryStreamRevisionsChunked(
+            backing)
+        repo = self.make_repository('.')
+        rev_id1_utf8 = u'\xc8'.encode('utf-8')
+        response = request.execute(backing.local_abspath(''))
+        self.assertEqual(None, response)
+        response = request.do_body("%s\n\n1" % (rev_id1_utf8,))
+        self.assertEqual(
+            FailedSmartServerResponse(('NoSuchRevision', )),
+            response)
+
+
 class TestSmartServerIsReadonly(tests.TestCaseWithTransport):
 
     def test_is_readonly_no(self):
@@ -808,7 +966,10 @@ class TestHandlers(tests.TestCase):
             smart.branch.SmartServerBranchRequestUnlock)
         self.assertEqual(
             smart.request.request_handlers.get('BzrDir.find_repository'),
-            smart.bzrdir.SmartServerRequestFindRepository)
+            smart.bzrdir.SmartServerRequestFindRepositoryV1)
+        self.assertEqual(
+            smart.request.request_handlers.get('BzrDir.find_repositoryV2'),
+            smart.bzrdir.SmartServerRequestFindRepositoryV2)
         self.assertEqual(
             smart.request.request_handlers.get('BzrDirFormat.initialize'),
             smart.bzrdir.SmartServerRequestInitializeBzrDir)
@@ -819,7 +980,11 @@ class TestHandlers(tests.TestCase):
             smart.request.request_handlers.get('Repository.gather_stats'),
             smart.repository.SmartServerRepositoryGatherStats)
         self.assertEqual(
-            smart.request.request_handlers.get('Repository.get_revision_graph'),
+            smart.request.request_handlers.get('Repository.get_parent_map'),
+            smart.repository.SmartServerRepositoryGetParentMap)
+        self.assertEqual(
+            smart.request.request_handlers.get(
+                'Repository.get_revision_graph'),
             smart.repository.SmartServerRepositoryGetRevisionGraph)
         self.assertEqual(
             smart.request.request_handlers.get('Repository.has_revision'),
@@ -830,12 +995,12 @@ class TestHandlers(tests.TestCase):
         self.assertEqual(
             smart.request.request_handlers.get('Repository.lock_write'),
             smart.repository.SmartServerRepositoryLockWrite)
-        self.assertEqual(
-            smart.request.request_handlers.get('Repository.unlock'),
-            smart.repository.SmartServerRepositoryUnlock)
         self.assertEqual(
             smart.request.request_handlers.get('Repository.tarball'),
             smart.repository.SmartServerRepositoryTarball)
+        self.assertEqual(
+            smart.request.request_handlers.get('Repository.unlock'),
+            smart.repository.SmartServerRepositoryUnlock)
         self.assertEqual(
             smart.request.request_handlers.get('Transport.is_readonly'),
             smart.request.SmartServerIsReadonly)
