@@ -15,30 +15,25 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-import os
-import sys
-import stat
 from cStringIO import StringIO
 
 import bzrlib
 from bzrlib import (
     errors,
+    osutils,
     urlutils,
     )
-from bzrlib.errors import (ConnectionError,
-                           DependencyNotPresent,
+from bzrlib.errors import (DependencyNotPresent,
                            FileExists,
                            InvalidURLJoin,
                            NoSuchFile,
                            PathNotChild,
-                           TransportNotPossible,
-                           ConnectionError,
-                           DependencyNotPresent,
                            ReadError,
                            UnsupportedProtocol,
                            )
 from bzrlib.tests import TestCase, TestCaseInTempDir
-from bzrlib.transport import (_CoalescedOffset,
+from bzrlib.transport import (_clear_protocol_handlers,
+                              _CoalescedOffset,
                               ConnectedTransport,
                               _get_protocol_handlers,
                               _set_protocol_handlers,
@@ -47,16 +42,12 @@ from bzrlib.transport import (_CoalescedOffset,
                               LateReadError,
                               register_lazy_transport,
                               register_transport_proto,
-                              _clear_protocol_handlers,
                               Transport,
                               )
 from bzrlib.transport.chroot import ChrootServer
 from bzrlib.transport.memory import MemoryTransport
 from bzrlib.transport.local import (LocalTransport,
                                     EmulatedWin32LocalTransport)
-from bzrlib.transport.remote import (BZR_DEFAULT_PORT,
-                                     RemoteTCPTransport,
-                                     )
 
 
 # TODO: Should possibly split transport-specific tests into their own files.
@@ -76,15 +67,20 @@ class TestTransport(TestCase):
 
     def test_get_transport_modules(self):
         handlers = _get_protocol_handlers()
+        # don't pollute the current handlers
+        _clear_protocol_handlers()
         class SampleHandler(object):
             """I exist, isnt that enough?"""
         try:
             _clear_protocol_handlers()
             register_transport_proto('foo')
-            register_lazy_transport('foo', 'bzrlib.tests.test_transport', 'TestTransport.SampleHandler')
+            register_lazy_transport('foo', 'bzrlib.tests.test_transport',
+                                    'TestTransport.SampleHandler')
             register_transport_proto('bar')
-            register_lazy_transport('bar', 'bzrlib.tests.test_transport', 'TestTransport.SampleHandler')
-            self.assertEqual([SampleHandler.__module__, 'bzrlib.transport.chroot'],
+            register_lazy_transport('bar', 'bzrlib.tests.test_transport',
+                                    'TestTransport.SampleHandler')
+            self.assertEqual([SampleHandler.__module__,
+                              'bzrlib.transport.chroot'],
                              _get_transport_modules())
         finally:
             _set_protocol_handlers(handlers)
@@ -92,6 +88,8 @@ class TestTransport(TestCase):
     def test_transport_dependency(self):
         """Transport with missing dependency causes no error"""
         saved_handlers = _get_protocol_handlers()
+        # don't pollute the current handlers
+        _clear_protocol_handlers()
         try:
             register_transport_proto('foo')
             register_lazy_transport('foo', 'bzrlib.tests.test_transport',
@@ -154,11 +152,12 @@ class TestTransport(TestCase):
 
 
 class TestCoalesceOffsets(TestCase):
-    
-    def check(self, expected, offsets, limit=0, fudge=0):
+
+    def check(self, expected, offsets, limit=0, max_size=0, fudge=0):
         coalesce = Transport._coalesce_offsets
         exp = [_CoalescedOffset(*x) for x in expected]
-        out = list(coalesce(offsets, limit=limit, fudge_factor=fudge))
+        out = list(coalesce(offsets, limit=limit, fudge_factor=fudge,
+                            max_size=max_size))
         self.assertEqual(exp, out)
 
     def test_coalesce_empty(self):
@@ -171,7 +170,7 @@ class TestCoalesceOffsets(TestCase):
         self.check([(0, 10, [(0, 10)]),
                     (20, 10, [(0, 10)]),
                    ], [(0, 10), (20, 10)])
-            
+
     def test_coalesce_unsorted(self):
         self.check([(20, 10, [(0, 10)]),
                     (0, 10, [(0, 10)]),
@@ -181,6 +180,7 @@ class TestCoalesceOffsets(TestCase):
         self.check([(0, 20, [(0, 10), (10, 10)])],
                    [(0, 10), (10, 10)])
 
+    # XXX: scary, http.readv() can't handle that --vila20071209
     def test_coalesce_overlapped(self):
         self.check([(0, 15, [(0, 10), (5, 10)])],
                    [(0, 10), (5, 10)])
@@ -210,6 +210,20 @@ class TestCoalesceOffsets(TestCase):
                    ], [(10, 10), (30, 10), (100, 10)],
                    fudge=10
                   )
+    def test_coalesce_max_size(self):
+        self.check([(10, 20, [(0, 10), (10, 10)]),
+                    (30, 50, [(0, 50)]),
+                    # If one range is above max_size, it gets its own coalesced
+                    # offset
+                    (100, 80, [(0, 80),]),],
+                   [(10, 10), (20, 10), (30, 50), (100, 80)],
+                   max_size=50
+                  )
+
+    def test_coalesce_no_max_size(self):
+        self.check([(10, 170, [(0, 10), (10, 10), (20, 50), (70, 100)]),],
+                   [(10, 10), (20, 10), (30, 50), (80, 100)],
+                  )
 
 
 class TestMemoryTransport(TestCase):
@@ -301,7 +315,6 @@ class TestMemoryTransport(TestCase):
     def test_parameters(self):
         transport = MemoryTransport()
         self.assertEqual(True, transport.listable())
-        self.assertEqual(False, transport.should_cache())
         self.assertEqual(False, transport.is_readonly())
 
     def test_iter_files_recursive(self):
@@ -417,13 +430,12 @@ class ReadonlyDecoratorTransportTest(TestCase):
         # connect to . in readonly mode
         transport = readonly.ReadonlyTransportDecorator('readonly+.')
         self.assertEqual(True, transport.listable())
-        self.assertEqual(False, transport.should_cache())
         self.assertEqual(True, transport.is_readonly())
 
     def test_http_parameters(self):
-        from bzrlib.tests.HttpServer import HttpServer
+        from bzrlib.tests.http_server import HttpServer
         import bzrlib.transport.readonly as readonly
-        # connect to . via http which is not listable
+        # connect to '.' via http which is not listable
         server = HttpServer()
         server.setUp()
         try:
@@ -431,7 +443,6 @@ class ReadonlyDecoratorTransportTest(TestCase):
             self.failUnless(isinstance(transport,
                                        readonly.ReadonlyTransportDecorator))
             self.assertEqual(False, transport.listable())
-            self.assertEqual(True, transport.should_cache())
             self.assertEqual(True, transport.is_readonly())
         finally:
             server.tearDown()
@@ -446,18 +457,17 @@ class FakeNFSDecoratorTests(TestCaseInTempDir):
         return fakenfs.FakeNFSTransportDecorator('fakenfs+' + url)
 
     def test_local_parameters(self):
-        # the listable, should_cache and is_readonly parameters
+        # the listable and is_readonly parameters
         # are not changed by the fakenfs decorator
         transport = self.get_nfs_transport('.')
         self.assertEqual(True, transport.listable())
-        self.assertEqual(False, transport.should_cache())
         self.assertEqual(False, transport.is_readonly())
 
     def test_http_parameters(self):
-        # the listable, should_cache and is_readonly parameters
+        # the listable and is_readonly parameters
         # are not changed by the fakenfs decorator
-        from bzrlib.tests.HttpServer import HttpServer
-        # connect to . via http which is not listable
+        from bzrlib.tests.http_server import HttpServer
+        # connect to '.' via http which is not listable
         server = HttpServer()
         server.setUp()
         try:
@@ -465,7 +475,6 @@ class FakeNFSDecoratorTests(TestCaseInTempDir):
             self.assertIsInstance(
                 transport, bzrlib.transport.fakenfs.FakeNFSTransportDecorator)
             self.assertEqual(False, transport.listable())
-            self.assertEqual(True, transport.should_cache())
             self.assertEqual(True, transport.is_readonly())
         finally:
             server.tearDown()
@@ -580,26 +589,26 @@ class TestTransportImplementation(TestCaseInTempDir):
 class TestLocalTransports(TestCase):
 
     def test_get_transport_from_abspath(self):
-        here = os.path.abspath('.')
+        here = osutils.abspath('.')
         t = get_transport(here)
         self.assertIsInstance(t, LocalTransport)
         self.assertEquals(t.base, urlutils.local_path_to_url(here) + '/')
 
     def test_get_transport_from_relpath(self):
-        here = os.path.abspath('.')
+        here = osutils.abspath('.')
         t = get_transport('.')
         self.assertIsInstance(t, LocalTransport)
         self.assertEquals(t.base, urlutils.local_path_to_url('.') + '/')
 
     def test_get_transport_from_local_url(self):
-        here = os.path.abspath('.')
+        here = osutils.abspath('.')
         here_url = urlutils.local_path_to_url(here) + '/'
         t = get_transport(here_url)
         self.assertIsInstance(t, LocalTransport)
         self.assertEquals(t.base, here_url)
 
     def test_local_abspath(self):
-        here = os.path.abspath('.')
+        here = osutils.abspath('.')
         t = get_transport(here)
         self.assertEquals(t.local_abspath(''), here)
 
@@ -623,14 +632,14 @@ class TestConnectedTransport(TestCase):
     """Tests for connected to remote server transports"""
 
     def test_parse_url(self):
-        t = ConnectedTransport('sftp://simple.example.com/home/source')
+        t = ConnectedTransport('http://simple.example.com/home/source')
         self.assertEquals(t._host, 'simple.example.com')
         self.assertEquals(t._port, None)
         self.assertEquals(t._path, '/home/source/')
         self.failUnless(t._user is None)
         self.failUnless(t._password is None)
 
-        self.assertEquals(t.base, 'sftp://simple.example.com/home/source/')
+        self.assertEquals(t.base, 'http://simple.example.com/home/source/')
 
     def test_parse_quoted_url(self):
         t = ConnectedTransport('http://ro%62ey:h%40t@ex%41mple.com:2222/path')
@@ -669,11 +678,13 @@ class TestConnectedTransport(TestCase):
         self.assertEquals(t.relpath('sftp://host.com/dev/%path/sub'), 'sub')
 
     def test_connection_sharing_propagate_credentials(self):
-        t = ConnectedTransport('foo://user@host.com/abs/path')
+        t = ConnectedTransport('ftp://user@host.com/abs/path')
+        self.assertEquals('user', t._user)
+        self.assertEquals('host.com', t._host)
         self.assertIs(None, t._get_connection())
         self.assertIs(None, t._password)
         c = t.clone('subdir')
-        self.assertEquals(None, c._get_connection())
+        self.assertIs(None, c._get_connection())
         self.assertIs(None, t._password)
 
         # Simulate the user entering a password
@@ -698,7 +709,10 @@ class TestReusedTransports(TestCase):
     """Tests for transport reuse"""
 
     def test_reuse_same_transport(self):
-        t1 = get_transport('http://foo/')
+        possible_transports = []
+        t1 = get_transport('http://foo/',
+                           possible_transports=possible_transports)
+        self.assertEqual([t1], possible_transports)
         t2 = get_transport('http://foo/', possible_transports=[t1])
         self.assertIs(t1, t2)
 
@@ -717,36 +731,44 @@ class TestReusedTransports(TestCase):
         self.assertIsNot(t1, t2)
 
 
-class TestRemoteTCPTransport(TestCase):
-    """Tests for bzr:// transport (RemoteTCPTransport)."""
+class TestTransportTrace(TestCase):
 
-    def test_relpath_with_implicit_port(self):
-        """Connected transports with the same URL are the same, even if the
-        port is implicit.
+    def test_get(self):
+        transport = get_transport('trace+memory://')
+        self.assertIsInstance(
+            transport, bzrlib.transport.trace.TransportTraceDecorator)
 
-        So t.relpath(url) should always be '' if t.base is the same as url, or
-        if the only difference is that one explicitly specifies the default
-        port and the other doesn't specify a port.
-        """
-        t_implicit_port = RemoteTCPTransport('bzr://host.com/')
-        self.assertEquals('', t_implicit_port.relpath('bzr://host.com/'))
-        t_explicit_port = RemoteTCPTransport('bzr://host.com:4155/')
-        self.assertEquals('', t_explicit_port.relpath('bzr://host.com:4155/'))
+    def test_clone_preserves_activity(self):
+        transport = get_transport('trace+memory://')
+        transport2 = transport.clone('.')
+        self.assertTrue(transport is not transport2)
+        self.assertTrue(transport._activity is transport2._activity)
 
-    def test_url_includes_non_default_port(self):
-        """Non-default ports are included in the transport's URL.
+    # the following specific tests are for the operations that have made use of
+    # logging in tests; we could test every single operation but doing that
+    # still won't cause a test failure when the top level Transport API
+    # changes; so there is little return doing that.
+    def test_get(self):
+        transport = get_transport('trace+memory:///')
+        transport.put_bytes('foo', 'barish')
+        transport.get('foo')
+        expected_result = []
+        # put_bytes records the bytes, not the content to avoid memory
+        # pressure.
+        expected_result.append(('put_bytes', 'foo', 6, None))
+        # get records the file name only.
+        expected_result.append(('get', 'foo'))
+        self.assertEqual(expected_result, transport._activity)
 
-        Contrast this to `test_url_omits_default_port`.
-        """
-        t = get_transport('bzr://host.com:666/')
-        self.assertEquals('bzr://host.com:666/', t.base)
-
-
-def get_test_permutations():
-    """Return transport permutations to be used in testing.
-
-    This module registers some transports, but they're only for testing
-    registration.  We don't really want to run all the transport tests against
-    them.
-    """
-    return []
+    def test_readv(self):
+        transport = get_transport('trace+memory:///')
+        transport.put_bytes('foo', 'barish')
+        list(transport.readv('foo', [(0, 1), (3, 2)], adjust_for_latency=True,
+            upper_limit=6))
+        expected_result = []
+        # put_bytes records the bytes, not the content to avoid memory
+        # pressure.
+        expected_result.append(('put_bytes', 'foo', 6, None))
+        # readv records the supplied offset request
+        expected_result.append(('readv', 'foo', [(0, 1), (3, 2)], True, 6))
+        self.assertEqual(expected_result, transport._activity)
