@@ -167,6 +167,21 @@ class TestMove(TestCaseWithWorkingTree):
                                ('d', 'd-id')], tree.basis_tree())
         tree._validate()
 
+    def test_move_over_deleted(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b', 'b'])
+        tree.add(['a', 'a/b', 'b'], ['a-id', 'ab-id', 'b-id'])
+        tree.commit('initial', rev_id='rev-1')
+
+        root_id = tree.get_root_id()
+        tree.remove(['a/b'], keep_files=False)
+        self.assertEqual([('b', 'a/b')], tree.move(['b'], 'a'))
+        self.assertTreeLayout([('', root_id),
+                               ('a', 'a-id'),
+                               ('a/b', 'b-id'),
+                              ], tree)
+        tree._validate()
+
     def test_move_subdir(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/c'])
@@ -358,6 +373,180 @@ class TestMove(TestCaseWithWorkingTree):
                                ('a/c/d', 'd-id')], tree.basis_tree())
         tree._validate()
 
+    def test_move_directory_into_parent(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['c/', 'c/b/', 'c/b/d/'])
+        tree.add(['c', 'c/b', 'c/b/d'],
+                 ['c-id', 'b-id', 'd-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+
+        self.assertEqual([('c/b', 'b')],
+                         tree.move(['c/b'], ''))
+        self.assertTreeLayout([('', root_id),
+                               ('b', 'b-id'),
+                               ('c', 'c-id'),
+                               ('b/d', 'd-id'),
+                              ], tree)
+        tree._validate()
+
+    def test_move_directory_with_children_in_subdir(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b', 'a/c/', 'd/'])
+        tree.add(['a', 'a/b', 'a/c', 'd'],
+                 ['a-id', 'b-id', 'c-id', 'd-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+
+
+        tree.rename_one('a/b', 'a/c/b')
+        self.assertTreeLayout([('', root_id),
+                               ('a', 'a-id'),
+                               ('d', 'd-id'),
+                               ('a/c', 'c-id'),
+                               ('a/c/b', 'b-id'),
+                              ], tree)
+        self.assertEqual([('a', 'd/a')],
+                         tree.move(['a'], 'd'))
+        self.assertTreeLayout([('', root_id),
+                               ('d', 'd-id'),
+                               ('d/a', 'a-id'),
+                               ('d/a/c', 'c-id'),
+                               ('d/a/c/b', 'b-id'),
+                              ], tree)
+        tree._validate()
+
+    def test_move_directory_with_deleted_children(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b', 'a/c', 'a/d', 'b/'])
+        tree.add(['a', 'b', 'a/b', 'a/c', 'a/d'],
+                 ['a-id', 'b-id', 'ab-id', 'ac-id', 'ad-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+
+        tree.remove(['a/b', 'a/d'])
+
+        self.assertEqual([('a', 'b/a')],
+                         tree.move(['a'], 'b'))
+        self.assertTreeLayout([('', root_id),
+                               ('b', 'b-id'),
+                               ('b/a', 'a-id'),
+                               ('b/a/c', 'ac-id'),
+                              ], tree)
+        tree._validate()
+
+    def test_move_directory_with_new_children(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/c', 'b/'])
+        tree.add(['a', 'b', 'a/c'], ['a-id', 'b-id', 'ac-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+
+        self.build_tree(['a/b', 'a/d'])
+        tree.add(['a/b', 'a/d'], ['ab-id', 'ad-id'])
+
+        self.assertEqual([('a', 'b/a')],
+                         tree.move(['a'], 'b'))
+        self.assertTreeLayout([('', root_id),
+                               ('b', 'b-id'),
+                               ('b/a', 'a-id'),
+                               ('b/a/b', 'ab-id'),
+                               ('b/a/c', 'ac-id'),
+                               ('b/a/d', 'ad-id'),
+                              ], tree)
+        tree._validate()
+
+    def test_move_directory_with_moved_children(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b', 'a/c', 'd', 'e/'])
+        tree.add(['a', 'a/b', 'a/c', 'd', 'e'],
+                 ['a-id', 'b-id', 'c-id', 'd-id', 'e-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+
+        self.assertEqual([('a/b', 'b')],
+                         tree.move(['a/b'], ''))
+        self.assertTreeLayout([('', root_id),
+                               ('a', 'a-id'),
+                               ('b', 'b-id'),
+                               ('d', 'd-id'),
+                               ('e', 'e-id'),
+                               ('a/c', 'c-id'),
+                              ], tree)
+        self.assertEqual([('d', 'a/d')],
+                         tree.move(['d'], 'a'))
+        self.assertTreeLayout([('', root_id),
+                               ('a', 'a-id'),
+                               ('b', 'b-id'),
+                               ('e', 'e-id'),
+                               ('a/c', 'c-id'),
+                               ('a/d', 'd-id'),
+                              ], tree)
+        self.assertEqual([('a', 'e/a')],
+                         tree.move(['a'], 'e'))
+        self.assertTreeLayout([('', root_id),
+                               ('b', 'b-id'),
+                               ('e', 'e-id'),
+                               ('e/a', 'a-id'),
+                               ('e/a/c', 'c-id'),
+                               ('e/a/d', 'd-id'),
+                              ], tree)
+        tree._validate()
+
+    def test_move_directory_with_renamed_child(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b', 'a/c', 'd/'])
+        tree.add(['a', 'a/b', 'a/c', 'd'],
+                 ['a-id', 'b-id', 'c-id', 'd-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+
+        tree.rename_one('a/b', 'a/d')
+        self.assertTreeLayout([('', root_id),
+                               ('a', 'a-id'),
+                               ('d', 'd-id'),
+                               ('a/c', 'c-id'),
+                               ('a/d', 'b-id'),
+                              ], tree)
+        self.assertEqual([('a', 'd/a')],
+                         tree.move(['a'], 'd'))
+        self.assertTreeLayout([('', root_id),
+                               ('d', 'd-id'),
+                               ('d/a', 'a-id'),
+                               ('d/a/c', 'c-id'),
+                               ('d/a/d', 'b-id'),
+                              ], tree)
+        tree._validate()
+
+    def test_move_directory_with_swapped_children(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b', 'a/c', 'a/d', 'e/'])
+        tree.add(['a', 'a/b', 'a/c', 'a/d', 'e'],
+                 ['a-id', 'b-id', 'c-id', 'd-id', 'e-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+
+        tree.rename_one('a/b', 'a/bb')
+        tree.rename_one('a/d', 'a/b')
+        tree.rename_one('a/bb', 'a/d')
+        self.assertTreeLayout([('', root_id),
+                               ('a', 'a-id'),
+                               ('e', 'e-id'),
+                               ('a/b', 'd-id'),
+                               ('a/c', 'c-id'),
+                               ('a/d', 'b-id'),
+                              ], tree)
+        self.assertEqual([('a', 'e/a')],
+                         tree.move(['a'], 'e'))
+        self.assertTreeLayout([('', root_id),
+                               ('e', 'e-id'),
+                               ('e/a', 'a-id'),
+                               ('e/a/b', 'd-id'),
+                               ('e/a/c', 'c-id'),
+                               ('e/a/d', 'b-id'),
+                              ], tree)
+        tree._validate()
+
     def test_move_moved(self):
         """Moving a moved entry works as expected."""
         tree = self.make_branch_and_tree('.')
