@@ -37,10 +37,13 @@ class TransportDecorator(Transport):
     method to return the url prefix for the subclass.
     """
 
-    def __init__(self, url, _decorated=None):
-        """Set the 'base' path where files will be stored.
+    def __init__(self, url, _decorated=None, _from_transport=None):
+        """Set the 'base' path of the transport.
         
-        _decorated is a private parameter for cloning."""
+        :param _decorated: A private parameter for cloning.
+        :param _from_transport: Is available for subclasses that
+            need to share state across clones.
+        """
         prefix = self._get_url_prefix()
         assert url.startswith(prefix), \
                 "url %r doesn't start with decorator prefix %r" % \
@@ -65,11 +68,16 @@ class TransportDecorator(Transport):
         """See Transport.append_bytes()."""
         return self._decorated.append_bytes(relpath, bytes, mode=mode)
 
+    def _can_roundtrip_unix_modebits(self):
+        """See Transport._can_roundtrip_unix_modebits()."""
+        return self._decorated._can_roundtrip_unix_modebits()
+
     def clone(self, offset=None):
         """See Transport.clone()."""
         decorated_clone = self._decorated.clone(offset)
         return self.__class__(
-            self._get_url_prefix() + decorated_clone.base, decorated_clone)
+            self._get_url_prefix() + decorated_clone.base, decorated_clone,
+            self)
 
     def delete(self, relpath):
         """See Transport.delete()."""
@@ -110,6 +118,10 @@ class TransportDecorator(Transport):
         """See Transport.mkdir()."""
         return self._decorated.mkdir(relpath, mode)
 
+    def open_write_stream(self, relpath, mode=None):
+        """See Transport.open_write_stream."""
+        return self._decorated.open_write_stream(relpath, mode=mode)
+
     def put_file(self, relpath, f, mode=None):
         """See Transport.put_file()."""
         return self._decorated.put_file(relpath, f, mode)
@@ -130,6 +142,14 @@ class TransportDecorator(Transport):
         """See Transport.list_dir()."""
         return self._decorated.list_dir(relpath)
 
+    def _readv(self, relpath, offsets):
+        """See Transport._readv."""
+        return self._decorated._readv(relpath, offsets)
+
+    def recommended_page_size(self):
+        """See Transport.recommended_page_size()."""
+        return self._decorated.recommended_page_size()
+
     def rename(self, rel_from, rel_to):
         return self._decorated.rename(rel_from, rel_to)
     
@@ -137,10 +157,6 @@ class TransportDecorator(Transport):
         """See Transport.rmdir."""
         return self._decorated.rmdir(relpath)
 
-    def should_cache(self):
-        """See Transport.should_cache()."""
-        return self._decorated.should_cache()
-
     def stat(self, relpath):
         """See Transport.stat()."""
         return self._decorated.stat(relpath)
