@@ -26,6 +26,7 @@ from bzrlib import (
     delta,
     osutils,
     revision as _mod_revision,
+    conflicts as _mod_conflicts,
     symbol_versioning,
     )
 from bzrlib.decorators import needs_read_lock
@@ -92,11 +93,15 @@ class Tree(object):
             want_unversioned=want_unversioned,
             )
 
-    def _iter_changes(self, from_tree, include_unchanged=False,
+    @symbol_versioning.deprecated_method(symbol_versioning.one_three)
+    def _iter_changes(self, *args, **kwargs):
+        return self.iter_changes(*args, **kwargs)
+
+    def iter_changes(self, from_tree, include_unchanged=False,
                      specific_files=None, pb=None, extra_trees=None,
                      require_versioned=True, want_unversioned=False):
         intertree = InterTree.get(from_tree, self)
-        return intertree._iter_changes(include_unchanged, specific_files, pb,
+        return intertree.iter_changes(include_unchanged, specific_files, pb,
             extra_trees, require_versioned, want_unversioned=want_unversioned)
     
     def conflicts(self):
@@ -104,7 +109,7 @@ class Tree(object):
 
         Each conflict is an instance of bzrlib.conflicts.Conflict.
         """
-        return []
+        return _mod_conflicts.ConflictList()
 
     def extras(self):
         """For trees that can have unversioned files, return all such paths."""
@@ -121,16 +126,14 @@ class Tree(object):
     
     def has_filename(self, filename):
         """True if the tree has given filename."""
-        raise NotImplementedError()
+        raise NotImplementedError(self.has_filename)
 
     def has_id(self, file_id):
-        file_id = osutils.safe_file_id(file_id)
         return self.inventory.has_id(file_id)
 
     __contains__ = has_id
 
     def has_or_had_id(self, file_id):
-        file_id = osutils.safe_file_id(file_id)
         if file_id == self.inventory.root.file_id:
             return True
         return self.inventory.has_id(file_id)
@@ -146,12 +149,15 @@ class Tree(object):
     def __iter__(self):
         return iter(self.inventory)
 
+    def all_file_ids(self):
+        """Iterate through all file ids, including ids for missing files."""
+        return set(self.inventory)
+
     def id2path(self, file_id):
         """Return the path for a file id.
 
         :raises NoSuchId:
         """
-        file_id = osutils.safe_file_id(file_id)
         return self.inventory.id2path(file_id)
 
     def is_control_filename(self, filename):
@@ -187,6 +193,28 @@ class Tree(object):
         raise NotImplementedError("Tree subclass %s must implement kind"
             % self.__class__.__name__)
 
+    def stored_kind(self, file_id):
+        """File kind stored for this file_id.
+
+        May not match kind on disk for working trees.  Always available
+        for versioned files, even when the file itself is missing.
+        """
+        return self.kind(file_id)
+
+    def path_content_summary(self, path):
+        """Get a summary of the information about path.
+        
+        :param path: A relative path within the tree.
+        :return: A tuple containing kind, size, exec, sha1-or-link.
+            Kind is always present (see tree.kind()).
+            size is present if kind is file, None otherwise.
+            exec is None unless kind is file and the platform supports the 'x'
+                bit.
+            sha1-or-link is the link target if kind is symlink, or the sha1 if
+                it can be obtained without reading the file.
+        """
+        raise NotImplementedError(self.path_content_summary)
+
     def get_reference_revision(self, file_id, path=None):
         raise NotImplementedError("Tree subclass %s must implement "
                                   "get_reference_revision"
@@ -209,8 +237,12 @@ class Tree(object):
     def _get_inventory(self):
         return self._inventory
     
-    def get_file(self, file_id):
-        """Return a file object for the file file_id in the tree."""
+    def get_file(self, file_id, path=None):
+        """Return a file object for the file file_id in the tree.
+        
+        If both file_id and path are defined, it is implementation defined as
+        to which one is used.
+        """
         raise NotImplementedError(self.get_file)
 
     def get_file_mtime(self, file_id, path=None):
@@ -223,7 +255,33 @@ class Tree(object):
         raise NotImplementedError(self.get_file_mtime)
 
     def get_file_by_path(self, path):
-        return self.get_file(self._inventory.path2id(path))
+        return self.get_file(self._inventory.path2id(path), path)
+
+    def iter_files_bytes(self, desired_files):
+        """Iterate through file contents.
+
+        Files will not necessarily be returned in the order they occur in
+        desired_files.  No specific order is guaranteed.
+
+        Yields pairs of identifier, bytes_iterator.  identifier is an opaque
+        value supplied by the caller as part of desired_files.  It should
+        uniquely identify the file version in the caller's context.  (Examples:
+        an index number or a TreeTransform trans_id.)
+
+        bytes_iterator is an iterable of bytestrings for the file.  The
+        kind of iterable and length of the bytestrings are unspecified, but for
+        this implementation, it is a tuple containing a single bytestring with
+        the complete text of the file.
+
+        :param desired_files: a list of (file_id, identifier) pairs
+        """
+        for file_id, identifier in desired_files:
+            # We wrap the string in a tuple so that we can return an iterable
+            # of bytestrings.  (Technically, a bytestring is also an iterable
+            # of bytestrings, but iterating through each character is not
+            # performant.)
+            cur_file = (self.get_file_text(file_id),)
+            yield identifier, cur_file
 
     def get_symlink_target(self, file_id):
         """Get the target for a given file_id.
@@ -235,32 +293,89 @@ class Tree(object):
         """
         raise NotImplementedError(self.get_symlink_target)
 
-    def annotate_iter(self, file_id):
-        """Return an iterator of revision_id, line tuples
+    def get_root_id(self):
+        """Return the file_id for the root of this tree."""
+        raise NotImplementedError(self.get_root_id)
+
+    def annotate_iter(self, file_id,
+                      default_revision=_mod_revision.CURRENT_REVISION):
+        """Return an iterator of revision_id, line tuples.
 
         For working trees (and mutable trees in general), the special
         revision_id 'current:' will be used for lines that are new in this
         tree, e.g. uncommitted changes.
         :param file_id: The file to produce an annotated version from
+        :param default_revision: For lines that don't match a basis, mark them
+            with this revision id. Not all implementations will make use of
+            this value.
         """
         raise NotImplementedError(self.annotate_iter)
 
-    def plan_file_merge(self, file_id, other):
-        """Generate a merge plan based on annotations
+    def _get_plan_merge_data(self, file_id, other, base):
+        from bzrlib import merge, versionedfile
+        vf = versionedfile._PlanMergeVersionedFile(file_id)
+        last_revision_a = self._get_file_revision(file_id, vf, 'this:')
+        last_revision_b = other._get_file_revision(file_id, vf, 'other:')
+        if base is None:
+            last_revision_base = None
+        else:
+            last_revision_base = base._get_file_revision(file_id, vf, 'base:')
+        return vf, last_revision_a, last_revision_b, last_revision_base
+
+    def plan_file_merge(self, file_id, other, base=None):
+        """Generate a merge plan based on annotations.
 
         If the file contains uncommitted changes in this tree, they will be
         attributed to the 'current:' pseudo-revision.  If the file contains
         uncommitted changes in the other tree, they will be assigned to the
         'other:' pseudo-revision.
         """
-        from bzrlib import merge
-        annotated_a = list(self.annotate_iter(file_id,
-                                              _mod_revision.CURRENT_REVISION))
-        annotated_b = list(other.annotate_iter(file_id, 'other:'))
-        ancestors_a = self._get_ancestors(_mod_revision.CURRENT_REVISION)
-        ancestors_b = other._get_ancestors('other:')
-        return merge._plan_annotate_merge(annotated_a, annotated_b,
-                                          ancestors_a, ancestors_b)
+        data = self._get_plan_merge_data(file_id, other, base)
+        vf, last_revision_a, last_revision_b, last_revision_base = data
+        return vf.plan_merge(last_revision_a, last_revision_b,
+                             last_revision_base)
+
+    def plan_file_lca_merge(self, file_id, other, base=None):
+        """Generate a merge plan based lca-newness.
+
+        If the file contains uncommitted changes in this tree, they will be
+        attributed to the 'current:' pseudo-revision.  If the file contains
+        uncommitted changes in the other tree, they will be assigned to the
+        'other:' pseudo-revision.
+        """
+        data = self._get_plan_merge_data(file_id, other, base)
+        vf, last_revision_a, last_revision_b, last_revision_base = data
+        return vf.plan_lca_merge(last_revision_a, last_revision_b,
+                                 last_revision_base)
+
+    def _get_file_revision(self, file_id, vf, tree_revision):
+        def file_revision(revision_tree):
+            revision_tree.lock_read()
+            try:
+                return revision_tree.inventory[file_id].revision
+            finally:
+                revision_tree.unlock()
+
+        def iter_parent_trees():
+            for revision_id in self.get_parent_ids():
+                try:
+                    yield self.revision_tree(revision_id)
+                except:
+                    yield self.repository.revision_tree(revision_id)
+
+        if getattr(self, '_get_weave', None) is None:
+            last_revision = tree_revision
+            parent_revisions = [file_revision(t) for t in iter_parent_trees()]
+            vf.add_lines(last_revision, parent_revisions,
+                         self.get_file(file_id).readlines())
+            repo = self.branch.repository
+            transaction = repo.get_transaction()
+            base_vf = repo.weave_store.get_weave(file_id, transaction)
+        else:
+            last_revision = file_revision(self)
+            base_vf = self._get_weave(file_id)
+        vf.fallback_versionedfiles.append(base_vf)
+        return last_revision
 
     inventory = property(_get_inventory,
                          doc="Inventory of this Tree")
@@ -292,10 +407,10 @@ class Tree(object):
     def paths2ids(self, paths, trees=[], require_versioned=True):
         """Return all the ids that can be reached by walking from paths.
         
-        Each path is looked up in each this tree and any extras provided in
+        Each path is looked up in this tree and any extras provided in
         trees, and this is repeated recursively: the children in an extra tree
         of a directory that has been renamed under a provided path in this tree
-        are all returned, even if none exist until a provided path in this
+        are all returned, even if none exist under a provided path in this
         tree, and vice versa.
 
         :param paths: An iterable of paths to start converting to ids from.
@@ -310,7 +425,6 @@ class Tree(object):
 
     def print_file(self, file_id):
         """Print file with id `file_id` to stdout."""
-        file_id = osutils.safe_file_id(file_id)
         import sys
         sys.stdout.write(self.get_file_text(file_id))
 
@@ -374,7 +488,7 @@ class Tree(object):
            versioned_kind.
          - lstat is the stat data *if* the file was statted.
          - path_from_tree_root is the path from the root of the tree.
-         - file_id is the file_id is the entry is versioned.
+         - file_id is the file_id if the entry is versioned.
          - versioned_kind is the kind of the file as last recorded in the 
            versioning system. If 'unknown' the file is not versioned.
         One of 'kind' and 'versioned_kind' must not be 'unknown'.
@@ -405,7 +519,6 @@ class EmptyTree(Tree):
         return False
 
     def kind(self, file_id):
-        file_id = osutils.safe_file_id(file_id)
         assert self._inventory[file_id].kind == "directory"
         return "directory"
 
@@ -413,7 +526,6 @@ class EmptyTree(Tree):
         return iter([])
     
     def __contains__(self, file_id):
-        file_id = osutils.safe_file_id(file_id)
         return (file_id in self._inventory)
 
     def get_file_sha1(self, file_id, path=None, stat_value=None):
@@ -514,7 +626,7 @@ def _find_ids_across_trees(filenames, trees, require_versioned):
     :param trees: The trees to find file_ids within
     :param require_versioned: if true, all specified filenames must occur in
         at least one tree.
-    :return: a set of (path, file ids) for the specified filenames
+    :return: a set of file ids for the specified filenames
     """
     not_versioned = []
     interesting_ids = set()
@@ -533,7 +645,7 @@ def _find_ids_across_trees(filenames, trees, require_versioned):
 
 
 def _find_children_across_trees(specified_ids, trees):
-    """Return a set including specified ids and their children
+    """Return a set including specified ids and their children.
     
     All matches in all trees will be used.
 
@@ -565,7 +677,7 @@ class InterTree(InterObject):
     Its instances have methods like 'compare' and contain references to the
     source and target trees these operations are to be carried out on.
 
-    clients of bzrlib should not need to use InterTree directly, rather they
+    Clients of bzrlib should not need to use InterTree directly, rather they
     should use the convenience methods on Tree such as 'Tree.compare()' which
     will pass through to InterTree as appropriate.
     """
@@ -614,7 +726,7 @@ class InterTree(InterObject):
             require_versioned=require_versioned,
             want_unversioned=want_unversioned)
 
-    def _iter_changes(self, include_unchanged=False,
+    def iter_changes(self, include_unchanged=False,
                       specific_files=None, pb=None, extra_trees=[],
                       require_versioned=True, want_unversioned=False):
         """Generate an iterator of changes between trees.
@@ -648,11 +760,15 @@ class InterTree(InterObject):
         lookup_trees = [self.source]
         if extra_trees:
              lookup_trees.extend(extra_trees)
-        specific_file_ids = self.target.paths2ids(specific_files,
-            lookup_trees, require_versioned=require_versioned)
+        if specific_files == []:
+            specific_file_ids = []
+        else:
+            specific_file_ids = self.target.paths2ids(specific_files,
+                lookup_trees, require_versioned=require_versioned)
         if want_unversioned:
-            all_unversioned = sorted([(p.split('/'), p) for p in self.target.extras()
-                if not specific_files or
+            all_unversioned = sorted([(p.split('/'), p) for p in
+                                     self.target.extras()
+                if specific_files is None or
                     osutils.is_inside_any(specific_files, p)])
             all_unversioned = deque(all_unversioned)
         else:
